@@ -5,7 +5,7 @@ import (
 	"strings"
 
 	"github.com/influxdata/influxdb/cmd/influx_tsm/tsdb/internal"
-	"github.com/influxdata/influxdb/influxql"
+	"github.com/influxdata/influxql"
 
 	"github.com/gogo/protobuf/proto"
 )
