@@ -1,3 +1,4 @@
+// Package help is the help subcommand of the influxd command.
 package help
 
 import (
@@ -41,5 +42,5 @@ The commands are:
 
 "run" is the default command.
 
-Use "influxd help [command]" for more information about a command.
+Use "influxd [command] -help" for more information about a command.
 `
