@@ -56,10 +56,10 @@ type StatementExecutor struct {
 }
 
 // ExecuteStatement executes the given statement with the given execution context.
-func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.ExecutionContext) error {
+func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 	// Select statements are handled separately so that they can be streamed.
 	if stmt, ok := stmt.(*influxql.SelectStatement); ok {
-		return e.executeSelectStatement(context.Background(), stmt, &ctx)
+		return e.executeSelectStatement(stmt, ctx)
 	}
 
 	var rows models.Rows
@@ -140,9 +140,9 @@ func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.
 		err = e.executeDropUserStatement(stmt)
 	case *influxql.ExplainStatement:
 		if stmt.Analyze {
-			rows, err = e.executeExplainAnalyzeStatement(stmt, &ctx)
+			rows, err = e.executeExplainAnalyzeStatement(stmt, ctx)
 		} else {
-			rows, err = e.executeExplainStatement(stmt, &ctx)
+			rows, err = e.executeExplainStatement(stmt, ctx)
 		}
 	case *influxql.GrantStatement:
 		if ctx.ReadOnly {
@@ -167,13 +167,13 @@ func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.
 	case *influxql.ShowContinuousQueriesStatement:
 		rows, err = e.executeShowContinuousQueriesStatement(stmt)
 	case *influxql.ShowDatabasesStatement:
-		rows, err = e.executeShowDatabasesStatement(stmt, &ctx)
+		rows, err = e.executeShowDatabasesStatement(stmt, ctx)
 	case *influxql.ShowDiagnosticsStatement:
 		rows, err = e.executeShowDiagnosticsStatement(stmt)
 	case *influxql.ShowGrantsForUserStatement:
 		rows, err = e.executeShowGrantsForUserStatement(stmt)
 	case *influxql.ShowMeasurementsStatement:
-		return e.executeShowMeasurementsStatement(stmt, &ctx)
+		return e.executeShowMeasurementsStatement(stmt, ctx)
 	case *influxql.ShowMeasurementCardinalityStatement:
 		rows, err = e.executeShowMeasurementCardinalityStatement(stmt)
 	case *influxql.ShowRetentionPoliciesStatement:
@@ -189,9 +189,9 @@ func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.
 	case *influxql.ShowSubscriptionsStatement:
 		rows, err = e.executeShowSubscriptionsStatement(stmt)
 	case *influxql.ShowTagKeysStatement:
-		return e.executeShowTagKeys(stmt, &ctx)
+		return e.executeShowTagKeys(stmt, ctx)
 	case *influxql.ShowTagValuesStatement:
-		return e.executeShowTagValues(stmt, &ctx)
+		return e.executeShowTagValues(stmt, ctx)
 	case *influxql.ShowUsersStatement:
 		rows, err = e.executeShowUsersStatement(stmt)
 	case *influxql.SetPasswordUserStatement:
@@ -211,9 +211,8 @@ func (e *StatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.
 	}
 
 	return ctx.Send(&query.Result{
-		StatementID: ctx.StatementID,
-		Series:      rows,
-		Messages:    messages,
+		Series:   rows,
+		Messages: messages,
 	})
 }
 
@@ -407,13 +406,12 @@ func (e *StatementExecutor) executeDropUserStatement(q *influxql.DropUserStateme
 	return e.MetaClient.DropUser(q.Name)
 }
 
-func (e *StatementExecutor) executeExplainStatement(q *influxql.ExplainStatement, ectx *query.ExecutionContext) (models.Rows, error) {
+func (e *StatementExecutor) executeExplainStatement(q *influxql.ExplainStatement, ctx *query.ExecutionContext) (models.Rows, error) {
 	opt := query.SelectOptions{
-		InterruptCh: ectx.InterruptCh,
-		NodeID:      ectx.ExecutionOptions.NodeID,
+		NodeID:      ctx.ExecutionOptions.NodeID,
 		MaxSeriesN:  e.MaxSelectSeriesN,
 		MaxBucketsN: e.MaxSelectBucketsN,
-		Authorizer:  ectx.Authorizer,
+		Authorizer:  ctx.Authorizer,
 	}
 
 	// Prepare the query for execution, but do not actually execute it.
@@ -442,13 +440,13 @@ func (e *StatementExecutor) executeExplainStatement(q *influxql.ExplainStatement
 func (e *StatementExecutor) executeExplainAnalyzeStatement(q *influxql.ExplainStatement, ectx *query.ExecutionContext) (models.Rows, error) {
 	stmt := q.Statement
 	t, span := tracing.NewTrace("select")
-	ctx := tracing.NewContextWithTrace(context.Background(), t)
+	ctx := tracing.NewContextWithTrace(ectx, t)
 	ctx = tracing.NewContextWithSpan(ctx, span)
 	var aux query.Iterators
 	ctx = query.NewContextWithIterators(ctx, &aux)
 	start := time.Now()
 
-	itrs, columns, err := e.createIterators(ctx, stmt, ectx)
+	cur, err := e.createIterators(ctx, stmt, ectx.ExecutionOptions)
 	if err != nil {
 		return nil, err
 	}
@@ -456,13 +454,7 @@ func (e *StatementExecutor) executeExplainAnalyzeStatement(q *influxql.ExplainSt
 	iterTime := time.Since(start)
 
 	// Generate a row emitter from the iterator set.
-	em := query.NewEmitter(itrs, stmt.TimeAscending(), ectx.ChunkSize)
-	em.Columns = columns
-	if stmt.Location != nil {
-		em.Location = stmt.Location
-	}
-	em.OmitTime = stmt.OmitTime
-	em.EmitName = stmt.EmitName
+	em := query.NewEmitter(cur, ectx.ChunkSize)
 
 	// Emit rows to the results channel.
 	var writeN int64
@@ -474,8 +466,8 @@ func (e *StatementExecutor) executeExplainAnalyzeStatement(q *influxql.ExplainSt
 		} else if row == nil {
 			// Check if the query was interrupted while emitting.
 			select {
-			case <-ectx.InterruptCh:
-				err = query.ErrQueryInterrupted
+			case <-ectx.Done():
+				err = ectx.Err()
 				goto CLEANUP
 			default:
 			}
@@ -544,20 +536,14 @@ func (e *StatementExecutor) executeSetPasswordUserStatement(q *influxql.SetPassw
 	return e.MetaClient.UpdateUser(q.Name, q.Password)
 }
 
-func (e *StatementExecutor) executeSelectStatement(ctx context.Context, stmt *influxql.SelectStatement, ectx *query.ExecutionContext) error {
-	itrs, columns, err := e.createIterators(ctx, stmt, ectx)
+func (e *StatementExecutor) executeSelectStatement(stmt *influxql.SelectStatement, ctx *query.ExecutionContext) error {
+	cur, err := e.createIterators(ctx, stmt, ctx.ExecutionOptions)
 	if err != nil {
 		return err
 	}
 
 	// Generate a row emitter from the iterator set.
-	em := query.NewEmitter(itrs, stmt.TimeAscending(), ectx.ChunkSize)
-	em.Columns = columns
-	if stmt.Location != nil {
-		em.Location = stmt.Location
-	}
-	em.OmitTime = stmt.OmitTime
-	em.EmitName = stmt.EmitName
+	em := query.NewEmitter(cur, ctx.ChunkSize)
 	defer em.Close()
 
 	// Emit rows to the results channel.
@@ -576,8 +562,8 @@ func (e *StatementExecutor) executeSelectStatement(ctx context.Context, stmt *in
 		} else if row == nil {
 			// Check if the query was interrupted while emitting.
 			select {
-			case <-ectx.InterruptCh:
-				return query.ErrQueryInterrupted
+			case <-ctx.Done():
+				return ctx.Err()
 			default:
 			}
 			break
@@ -593,13 +579,12 @@ func (e *StatementExecutor) executeSelectStatement(ctx context.Context, stmt *in
 		}
 
 		result := &query.Result{
-			StatementID: ectx.StatementID,
-			Series:      []*models.Row{row},
-			Partial:     partial,
+			Series:  []*models.Row{row},
+			Partial: partial,
 		}
 
 		// Send results or exit if closing.
-		if err := ectx.Send(result); err != nil {
+		if err := ctx.Send(result); err != nil {
 			return err
 		}
 
@@ -613,13 +598,12 @@ func (e *StatementExecutor) executeSelectStatement(ctx context.Context, stmt *in
 		}
 
 		var messages []*query.Message
-		if ectx.ReadOnly {
+		if ctx.ReadOnly {
 			messages = append(messages, query.ReadOnlyWarning(stmt.String()))
 		}
 
-		return ectx.Send(&query.Result{
-			StatementID: ectx.StatementID,
-			Messages:    messages,
+		return ctx.Send(&query.Result{
+			Messages: messages,
 			Series: []*models.Row{{
 				Name:    "result",
 				Columns: []string{"time", "written"},
@@ -630,35 +614,29 @@ func (e *StatementExecutor) executeSelectStatement(ctx context.Context, stmt *in
 
 	// Always emit at least one result.
 	if !emitted {
-		return ectx.Send(&query.Result{
-			StatementID: ectx.StatementID,
-			Series:      make([]*models.Row, 0),
+		return ctx.Send(&query.Result{
+			Series: make([]*models.Row, 0),
 		})
 	}
 
 	return nil
 }
 
-func (e *StatementExecutor) createIterators(ctx context.Context, stmt *influxql.SelectStatement, ectx *query.ExecutionContext) ([]query.Iterator, []string, error) {
-	opt := query.SelectOptions{
-		InterruptCh: ectx.InterruptCh,
-		NodeID:      ectx.ExecutionOptions.NodeID,
+func (e *StatementExecutor) createIterators(ctx context.Context, stmt *influxql.SelectStatement, opt query.ExecutionOptions) (query.Cursor, error) {
+	sopt := query.SelectOptions{
+		NodeID:      opt.NodeID,
 		MaxSeriesN:  e.MaxSelectSeriesN,
+		MaxPointN:   e.MaxSelectPointN,
 		MaxBucketsN: e.MaxSelectBucketsN,
-		Authorizer:  ectx.Authorizer,
+		Authorizer:  opt.Authorizer,
 	}
 
 	// Create a set of iterators from a selection.
-	itrs, columns, err := query.Select(ctx, stmt, e.ShardMapper, opt)
+	cur, err := query.Select(ctx, stmt, e.ShardMapper, sopt)
 	if err != nil {
-		return nil, nil, err
+		return nil, err
 	}
-
-	if e.MaxSelectPointN > 0 {
-		monitor := query.PointLimitMonitor(itrs, query.DefaultStatsInterval, e.MaxSelectPointN)
-		ectx.Query.Monitor(monitor)
-	}
-	return itrs, columns, nil
+	return cur, nil
 }
 
 func (e *StatementExecutor) executeShowContinuousQueriesStatement(stmt *influxql.ShowContinuousQueriesStatement) (models.Rows, error) {
@@ -738,8 +716,7 @@ func (e *StatementExecutor) executeShowMeasurementsStatement(q *influxql.ShowMea
 	names, err := e.TSDBStore.MeasurementNames(ctx.Authorizer, q.Database, q.Condition)
 	if err != nil || len(names) == 0 {
 		return ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-			Err:         err,
+			Err: err,
 		})
 	}
 
@@ -763,13 +740,10 @@ func (e *StatementExecutor) executeShowMeasurementsStatement(q *influxql.ShowMea
 	}
 
 	if len(values) == 0 {
-		return ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-		})
+		return ctx.Send(&query.Result{})
 	}
 
 	return ctx.Send(&query.Result{
-		StatementID: ctx.StatementID,
 		Series: []*models.Row{{
 			Name:    "measurements",
 			Columns: []string{"name"},
@@ -894,25 +868,39 @@ func (e *StatementExecutor) executeShowShardGroupsStatement(stmt *influxql.ShowS
 }
 
 func (e *StatementExecutor) executeShowStatsStatement(stmt *influxql.ShowStatsStatement) (models.Rows, error) {
-	stats, err := e.Monitor.Statistics(nil)
-	if err != nil {
-		return nil, err
-	}
-
 	var rows []*models.Row
-	for _, stat := range stats {
-		if stmt.Module != "" && stat.Name != stmt.Module {
-			continue
-		}
-		row := &models.Row{Name: stat.Name, Tags: stat.Tags}
 
-		values := make([]interface{}, 0, len(stat.Values))
-		for _, k := range stat.ValueNames() {
-			row.Columns = append(row.Columns, k)
-			values = append(values, stat.Values[k])
+	if _, ok := e.TSDBStore.(*tsdb.Store); stmt.Module == "indexes" && ok {
+		// The cost of collecting indexes metrics grows with the size of the indexes, so only collect this
+		// stat when explicitly requested.
+		b := e.TSDBStore.(*tsdb.Store).IndexBytes()
+		row := &models.Row{
+			Name:    "indexes",
+			Columns: []string{"memoryBytes"},
+			Values:  [][]interface{}{{b}},
 		}
-		row.Values = [][]interface{}{values}
 		rows = append(rows, row)
+
+	} else {
+		stats, err := e.Monitor.Statistics(nil)
+		if err != nil {
+			return nil, err
+		}
+
+		for _, stat := range stats {
+			if stmt.Module != "" && stat.Name != stmt.Module {
+				continue
+			}
+			row := &models.Row{Name: stat.Name, Tags: stat.Tags}
+
+			values := make([]interface{}, 0, len(stat.Values))
+			for _, k := range stat.ValueNames() {
+				row.Columns = append(row.Columns, k)
+				values = append(values, stat.Values[k])
+			}
+			row.Values = [][]interface{}{values}
+			rows = append(rows, row)
+		}
 	}
 	return rows, nil
 }
@@ -975,8 +963,7 @@ func (e *StatementExecutor) executeShowTagKeys(q *influxql.ShowTagKeysStatement,
 	tagKeys, err := e.TSDBStore.TagKeys(ctx.Authorizer, shardIDs, cond)
 	if err != nil {
 		return ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-			Err:         err,
+			Err: err,
 		})
 	}
 
@@ -1009,8 +996,7 @@ func (e *StatementExecutor) executeShowTagKeys(q *influxql.ShowTagKeysStatement,
 		}
 
 		if err := ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-			Series:      []*models.Row{row},
+			Series: []*models.Row{row},
 		}); err != nil {
 			return err
 		}
@@ -1019,9 +1005,7 @@ func (e *StatementExecutor) executeShowTagKeys(q *influxql.ShowTagKeysStatement,
 
 	// Ensure at least one result is emitted.
 	if !emitted {
-		return ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-		})
+		return ctx.Send(&query.Result{})
 	}
 	return nil
 }
@@ -1065,10 +1049,7 @@ func (e *StatementExecutor) executeShowTagValues(q *influxql.ShowTagValuesStatem
 
 	tagValues, err := e.TSDBStore.TagValues(ctx.Authorizer, shardIDs, cond)
 	if err != nil {
-		return ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-			Err:         err,
-		})
+		return ctx.Send(&query.Result{Err: err})
 	}
 
 	emitted := false
@@ -1103,8 +1084,7 @@ func (e *StatementExecutor) executeShowTagValues(q *influxql.ShowTagValuesStatem
 		}
 
 		if err := ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-			Series:      []*models.Row{row},
+			Series: []*models.Row{row},
 		}); err != nil {
 			return err
 		}
@@ -1113,9 +1093,7 @@ func (e *StatementExecutor) executeShowTagValues(q *influxql.ShowTagValuesStatem
 
 	// Ensure at least one result is emitted.
 	if !emitted {
-		return ctx.Send(&query.Result{
-			StatementID: ctx.StatementID,
-		})
+		return ctx.Send(&query.Result{})
 	}
 	return nil
 }
@@ -1216,7 +1194,7 @@ func (e *StatementExecutor) writeInto(w pointsWriter, stmt *influxql.SelectState
 	// convert rows back to points. The Executors (both aggregate and raw) are complex
 	// enough that changing them to write back to the DB is going to be clumsy
 	//
-	// it might seem weird to have the write be in the QueryExecutor, but the interweaving of
+	// it might seem weird to have the write be in the Executor, but the interweaving of
 	// limitedRowWriter and ExecuteAggregate/Raw makes it ridiculously hard to make sure that the
 	// results will be the same as when queried normally.
 	name := stmt.Target.Measurement.Name
@@ -1282,7 +1260,8 @@ func convertRowToPoints(measurementName string, row *models.Row) ([]models.Point
 }
 
 // NormalizeStatement adds a default database and policy to the measurements in statement.
-func (e *StatementExecutor) NormalizeStatement(stmt influxql.Statement, defaultDatabase string) (err error) {
+// Parameter defaultRetentionPolicy can be "".
+func (e *StatementExecutor) NormalizeStatement(stmt influxql.Statement, defaultDatabase, defaultRetentionPolicy string) (err error) {
 	influxql.WalkFunc(stmt, func(node influxql.Node) {
 		if err != nil {
 			return
@@ -1318,14 +1297,14 @@ func (e *StatementExecutor) NormalizeStatement(stmt influxql.Statement, defaultD
 				// DB and RP not supported by these statements so don't rewrite into invalid
 				// statements
 			default:
-				err = e.normalizeMeasurement(node, defaultDatabase)
+				err = e.normalizeMeasurement(node, defaultDatabase, defaultRetentionPolicy)
 			}
 		}
 	})
 	return
 }
 
-func (e *StatementExecutor) normalizeMeasurement(m *influxql.Measurement, defaultDatabase string) error {
+func (e *StatementExecutor) normalizeMeasurement(m *influxql.Measurement, defaultDatabase, defaultRetentionPolicy string) error {
 	// Targets (measurements in an INTO clause) can have blank names, which means it will be
 	// the same as the measurement name it came from in the FROM clause.
 	if !m.IsTarget && m.Name == "" && m.SystemIterator == "" && m.Regex == nil {
@@ -1350,10 +1329,13 @@ func (e *StatementExecutor) normalizeMeasurement(m *influxql.Measurement, defaul
 
 	// If no retention policy was specified, use the default.
 	if m.RetentionPolicy == "" {
-		if di.DefaultRetentionPolicy == "" {
+		if defaultRetentionPolicy != "" {
+			m.RetentionPolicy = defaultRetentionPolicy
+		} else if di.DefaultRetentionPolicy != "" {
+			m.RetentionPolicy = di.DefaultRetentionPolicy
+		} else {
 			return fmt.Errorf("default retention policy not set for: %s", di.Name)
 		}
-		m.RetentionPolicy = di.DefaultRetentionPolicy
 	}
 	return nil
 }
