@@ -20,6 +20,14 @@ func ErrRetentionPolicyNotFound(name string) error {
 	return fmt.Errorf("retention policy not found: %s", name)
 }
 
+// IsAuthorizationError indicates whether an error is due to an authorization failure
+func IsAuthorizationError(err error) bool {
+	e, ok := err.(interface {
+		AuthorizationFailed() bool
+	})
+	return ok && e.AuthorizationFailed()
+}
+
 // IsClientError indicates whether an error is a known client error.
 func IsClientError(err error) bool {
 	if err == nil {
@@ -32,13 +40,3 @@ func IsClientError(err error) bool {
 
 	return false
 }
-
-const upgradeMessage = `*******************************************************************
-                 UNSUPPORTED SHARD FORMAT DETECTED
-
-As of version 0.11, only tsm shards are supported. Please use the
-influx_tsm tool to convert non-tsm shards.
-
-More information can be found at the documentation site:
-https://docs.influxdata.com/influxdb/v0.10/administration/upgrading
-*******************************************************************`
