@@ -3,8 +3,8 @@ package internal
 import (
 	"time"
 
-	"github.com/influxdata/influxdb/influxql"
 	"github.com/influxdata/influxdb/services/meta"
+	"github.com/influxdata/influxql"
 )
 
 // MetaClientMock is a mockable implementation of meta.MetaClient.
@@ -32,6 +32,9 @@ type MetaClientMock struct {
 
 	OpenFn func() error
 
+	PrecreateShardGroupsFn func(from, to time.Time) error
+	PruneShardGroupsFn     func() error
+
 	RetentionPolicyFn func(database, name string) (rpi *meta.RetentionPolicyInfo, err error)
 
 	AuthenticateFn           func(username, password string) (ui meta.User, err error)
@@ -41,6 +44,7 @@ type MetaClientMock struct {
 	SetPrivilegeFn           func(username, database string, p influxql.Privilege) error
 	ShardGroupsByTimeRangeFn func(database, policy string, min, max time.Time) (a []meta.ShardGroupInfo, err error)
 	ShardOwnerFn             func(shardID uint64) (database, policy string, sgi *meta.ShardGroupInfo)
+	TruncateShardGroupsFn    func(t time.Time) error
 	UpdateRetentionPolicyFn  func(database, name string, rpu *meta.RetentionPolicyUpdate, makeDefault bool) error
 	UpdateUserFn             func(name, password string) error
 	UserPrivilegeFn          func(username, database string) (*influxql.Privilege, error)
@@ -137,6 +141,10 @@ func (c *MetaClientMock) ShardOwner(shardID uint64) (database, policy string, sg
 	return c.ShardOwnerFn(shardID)
 }
 
+func (c *MetaClientMock) TruncateShardGroups(t time.Time) error {
+	return c.TruncateShardGroupsFn(t)
+}
+
 func (c *MetaClientMock) UpdateRetentionPolicy(database, name string, rpu *meta.RetentionPolicyUpdate, makeDefault bool) error {
 	return c.UpdateRetentionPolicyFn(database, name, rpu, makeDefault)
 }
@@ -164,3 +172,8 @@ func (c *MetaClientMock) Users() []meta.UserInfo                  { return c.Use
 func (c *MetaClientMock) Open() error                { return c.OpenFn() }
 func (c *MetaClientMock) Data() meta.Data            { return c.DataFn() }
 func (c *MetaClientMock) SetData(d *meta.Data) error { return c.SetDataFn(d) }
+
+func (c *MetaClientMock) PrecreateShardGroups(from, to time.Time) error {
+	return c.PrecreateShardGroupsFn(from, to)
+}
+func (c *MetaClientMock) PruneShardGroups() error { return c.PruneShardGroupsFn() }
