@@ -13,10 +13,11 @@ import (
 	"sync"
 	"time"
 
+	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/monitor/diagnostics"
 	"github.com/influxdata/influxdb/services/meta"
-	"github.com/uber-go/zap"
+	"go.uber.org/zap"
 )
 
 // Policy constants.
@@ -61,7 +62,7 @@ type Monitor struct {
 	// Writer for pushing stats back into the database.
 	PointsWriter PointsWriter
 
-	Logger zap.Logger
+	Logger *zap.Logger
 }
 
 // PointsWriter is a simplified interface for writing the points the monitor gathers.
@@ -79,7 +80,7 @@ func New(r Reporter, c Config) *Monitor {
 		storeDatabase:        c.StoreDatabase,
 		storeInterval:        time.Duration(c.StoreInterval),
 		storeRetentionPolicy: MonitorRetentionPolicy,
-		Logger:               zap.New(zap.NullEncoder()),
+		Logger:               zap.NewNop(),
 	}
 }
 
@@ -98,7 +99,7 @@ func (m *Monitor) Open() error {
 		return nil
 	}
 
-	m.Logger.Info("Starting monitor system")
+	m.Logger.Info("Starting monitor service")
 
 	// Self-register various stats and diagnostics.
 	m.RegisterDiagnosticsClient("build", &build{
@@ -149,7 +150,7 @@ func (m *Monitor) writePoints(p models.Points) error {
 	defer m.mu.RUnlock()
 
 	if err := m.PointsWriter.WritePoints(m.storeDatabase, m.storeRetentionPolicy, p); err != nil {
-		m.Logger.Info(fmt.Sprintf("failed to store statistics: %s", err))
+		m.Logger.Info("failed to store statistics", zap.Error(err))
 	}
 	return nil
 }
@@ -157,11 +158,11 @@ func (m *Monitor) writePoints(p models.Points) error {
 // Close closes the monitor system.
 func (m *Monitor) Close() error {
 	if !m.open() {
-		m.Logger.Info("Monitor is already closed.")
+		m.Logger.Info("Monitor is already closed")
 		return nil
 	}
 
-	m.Logger.Info("shutting down monitor system")
+	m.Logger.Info("Shutting down monitor service")
 	m.mu.Lock()
 	close(m.done)
 	m.mu.Unlock()
@@ -211,7 +212,7 @@ func (m *Monitor) SetPointsWriter(pw PointsWriter) error {
 }
 
 // WithLogger sets the logger for the Monitor.
-func (m *Monitor) WithLogger(log zap.Logger) {
+func (m *Monitor) WithLogger(log *zap.Logger) {
 	m.Logger = log.With(zap.String("service", "monitor"))
 }
 
@@ -220,7 +221,7 @@ func (m *Monitor) RegisterDiagnosticsClient(name string, client diagnostics.Clie
 	m.mu.Lock()
 	defer m.mu.Unlock()
 	m.diagRegistrations[name] = client
-	m.Logger.Info(fmt.Sprintf(`'%s' registered for diagnostics monitoring`, name))
+	m.Logger.Info("Registered diagnostics client", zap.String("name", name))
 }
 
 // DeregisterDiagnosticsClient deregisters a diagnostics client by name.
@@ -250,8 +251,11 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*Statistic, error) {
 			statistic.Tags[k] = v
 		}
 
-		// Every other top-level expvar value is a map.
-		m := kv.Value.(*expvar.Map)
+		// Every other top-level expvar value should be a map.
+		m, ok := kv.Value.(*expvar.Map)
+		if !ok {
+			return
+		}
 
 		m.Do(func(subKV expvar.KeyValue) {
 			switch subKV.Key {
@@ -344,8 +348,10 @@ func (m *Monitor) gatherStatistics(statistics []*Statistic, tags map[string]stri
 	m.mu.RLock()
 	defer m.mu.RUnlock()
 
-	for _, s := range m.reporter.Statistics(tags) {
-		statistics = append(statistics, &Statistic{Statistic: s})
+	if m.reporter != nil {
+		for _, s := range m.reporter.Statistics(tags) {
+			statistics = append(statistics, &Statistic{Statistic: s})
+		}
 	}
 	return statistics
 }
@@ -384,8 +390,7 @@ func (m *Monitor) createInternalStorage() {
 		}
 
 		if _, err := m.MetaClient.CreateDatabaseWithRetentionPolicy(m.storeDatabase, &spec); err != nil {
-			m.Logger.Info(fmt.Sprintf("failed to create database '%s', failed to create storage: %s",
-				m.storeDatabase, err.Error()))
+			m.Logger.Info("Failed to create storage", logger.Database(m.storeDatabase), zap.Error(err))
 			return
 		}
 	}
@@ -412,8 +417,7 @@ func (m *Monitor) waitUntilInterval(d time.Duration) error {
 // storeStatistics writes the statistics to an InfluxDB system.
 func (m *Monitor) storeStatistics() {
 	defer m.wg.Done()
-	m.Logger.Info(fmt.Sprintf("Storing statistics in database '%s' retention policy '%s', at interval %s",
-		m.storeDatabase, m.storeRetentionPolicy, m.storeInterval))
+	m.Logger.Info("Storing statistics", logger.Database(m.storeDatabase), logger.RetentionPolicy(m.storeRetentionPolicy), logger.DurationLiteral("interval", m.storeInterval))
 
 	// Wait until an even interval to start recording monitor statistics.
 	// If we are interrupted before the interval for some reason, exit early.
@@ -436,7 +440,7 @@ func (m *Monitor) storeStatistics() {
 
 			stats, err := m.Statistics(m.globalTags)
 			if err != nil {
-				m.Logger.Info(fmt.Sprintf("failed to retrieve registered statistics: %s", err))
+				m.Logger.Info("Failed to retrieve registered statistics", zap.Error(err))
 				return
 			}
 
@@ -445,7 +449,7 @@ func (m *Monitor) storeStatistics() {
 			for _, s := range stats {
 				pt, err := models.NewPoint(s.Name, models.NewTags(s.Tags), s.Values, now)
 				if err != nil {
-					m.Logger.Info(fmt.Sprintf("Dropping point %v: %v", s.Name, err))
+					m.Logger.Info("Dropping point", zap.String("name", s.Name), zap.Error(err))
 					return
 				}
 				batch = append(batch, pt)
@@ -461,7 +465,7 @@ func (m *Monitor) storeStatistics() {
 				m.writePoints(batch)
 			}
 		case <-m.done:
-			m.Logger.Info(fmt.Sprintf("terminating storage of statistics"))
+			m.Logger.Info("Terminating storage of statistics")
 			return
 		}
 	}
