@@ -1,8 +1,9 @@
+// Package slices contains functions to operate on slices treated as sets.
 package slices // import "github.com/influxdata/influxdb/pkg/slices"
 
 import "strings"
 
-// Union combines two string sets
+// Union combines two string sets.
 func Union(setA, setB []string, ignoreCase bool) []string {
 	for _, b := range setB {
 		if ignoreCase {
@@ -18,7 +19,7 @@ func Union(setA, setB []string, ignoreCase bool) []string {
 	return setA
 }
 
-// Exists checks if a string is in a set
+// Exists checks if a string is in a set.
 func Exists(set []string, find string) bool {
 	for _, s := range set {
 		if s == find {
@@ -28,7 +29,7 @@ func Exists(set []string, find string) bool {
 	return false
 }
 
-// ExistsIgnoreCase checks if a string is in a set but ignores its case
+// ExistsIgnoreCase checks if a string is in a set but ignores its case.
 func ExistsIgnoreCase(set []string, find string) bool {
 	find = strings.ToLower(find)
 	for _, s := range set {
@@ -38,3 +39,12 @@ func ExistsIgnoreCase(set []string, find string) bool {
 	}
 	return false
 }
+
+// StringsToBytes converts a variable number of strings into a slice of []byte.
+func StringsToBytes(s ...string) [][]byte {
+	a := make([][]byte, 0, len(s))
+	for _, v := range s {
+		a = append(a, []byte(v))
+	}
+	return a
+}
