@@ -6,10 +6,83 @@ import (
 	"sort"
 	"time"
 
+	"github.com/influxdata/influxdb/query/internal/gota"
 	"github.com/influxdata/influxdb/query/neldermead"
 	"github.com/influxdata/influxql"
 )
 
+// FieldMapper is a FieldMapper that wraps another FieldMapper and exposes
+// the functions implemented by the query engine.
+type FieldMapper struct {
+	influxql.FieldMapper
+}
+
+func (m FieldMapper) CallType(name string, args []influxql.DataType) (influxql.DataType, error) {
+	if mapper, ok := m.FieldMapper.(influxql.CallTypeMapper); ok {
+		typ, err := mapper.CallType(name, args)
+		if err != nil {
+			return influxql.Unknown, err
+		} else if typ != influxql.Unknown {
+			return typ, nil
+		}
+	}
+
+	// Use the default FunctionTypeMapper for the query engine.
+	typmap := FunctionTypeMapper{}
+	return typmap.CallType(name, args)
+}
+
+// CallTypeMapper returns the types for call iterator functions.
+// Call iterator functions are commonly implemented within the storage engine
+// so this mapper is limited to only the return values of those functions.
+type CallTypeMapper struct{}
+
+func (CallTypeMapper) MapType(measurement *influxql.Measurement, field string) influxql.DataType {
+	return influxql.Unknown
+}
+
+func (CallTypeMapper) CallType(name string, args []influxql.DataType) (influxql.DataType, error) {
+	// If the function is not implemented by the embedded field mapper, then
+	// see if we implement the function and return the type here.
+	switch name {
+	case "mean":
+		return influxql.Float, nil
+	case "count":
+		return influxql.Integer, nil
+	case "min", "max", "sum", "first", "last":
+		// TODO(jsternberg): Verify the input type.
+		return args[0], nil
+	}
+	return influxql.Unknown, nil
+}
+
+// FunctionTypeMapper handles the type mapping for all functions implemented by the
+// query engine.
+type FunctionTypeMapper struct {
+	CallTypeMapper
+}
+
+func (FunctionTypeMapper) MapType(measurement *influxql.Measurement, field string) influxql.DataType {
+	return influxql.Unknown
+}
+
+func (m FunctionTypeMapper) CallType(name string, args []influxql.DataType) (influxql.DataType, error) {
+	if typ, err := m.CallTypeMapper.CallType(name, args); typ != influxql.Unknown || err != nil {
+		return typ, err
+	}
+
+	// Handle functions implemented by the query engine.
+	switch name {
+	case "median", "integral", "stddev":
+		return influxql.Float, nil
+	case "elapsed":
+		return influxql.Integer, nil
+	default:
+		// TODO(jsternberg): Do not use default for this.
+		return args[0], nil
+	}
+}
+
 // FloatMeanReducer calculates the mean of the aggregated points.
 type FloatMeanReducer struct {
 	sum   float64
@@ -103,6 +176,92 @@ func (r *UnsignedMeanReducer) Emit() []FloatPoint {
 	}}
 }
 
+type FloatSpreadReducer struct {
+	min, max float64
+	count    uint32
+}
+
+func NewFloatSpreadReducer() *FloatSpreadReducer {
+	return &FloatSpreadReducer{
+		min: math.Inf(1),
+		max: math.Inf(-1),
+	}
+}
+
+func (r *FloatSpreadReducer) AggregateFloat(p *FloatPoint) {
+	r.min = math.Min(r.min, p.Value)
+	r.max = math.Max(r.max, p.Value)
+	r.count++
+}
+
+func (r *FloatSpreadReducer) Emit() []FloatPoint {
+	return []FloatPoint{{
+		Time:       ZeroTime,
+		Value:      r.max - r.min,
+		Aggregated: r.count,
+	}}
+}
+
+type IntegerSpreadReducer struct {
+	min, max int64
+	count    uint32
+}
+
+func NewIntegerSpreadReducer() *IntegerSpreadReducer {
+	return &IntegerSpreadReducer{
+		min: math.MaxInt64,
+		max: math.MinInt64,
+	}
+}
+
+func (r *IntegerSpreadReducer) AggregateInteger(p *IntegerPoint) {
+	if p.Value < r.min {
+		r.min = p.Value
+	}
+	if p.Value > r.max {
+		r.max = p.Value
+	}
+	r.count++
+}
+
+func (r *IntegerSpreadReducer) Emit() []IntegerPoint {
+	return []IntegerPoint{{
+		Time:       ZeroTime,
+		Value:      r.max - r.min,
+		Aggregated: r.count,
+	}}
+}
+
+type UnsignedSpreadReducer struct {
+	min, max uint64
+	count    uint32
+}
+
+func NewUnsignedSpreadReducer() *UnsignedSpreadReducer {
+	return &UnsignedSpreadReducer{
+		min: math.MaxUint64,
+		max: 0,
+	}
+}
+
+func (r *UnsignedSpreadReducer) AggregateUnsigned(p *UnsignedPoint) {
+	if p.Value < r.min {
+		r.min = p.Value
+	}
+	if p.Value > r.max {
+		r.max = p.Value
+	}
+	r.count++
+}
+
+func (r *UnsignedSpreadReducer) Emit() []UnsignedPoint {
+	return []UnsignedPoint{{
+		Time:       ZeroTime,
+		Value:      r.max - r.min,
+		Aggregated: r.count,
+	}}
+}
+
 // FloatDerivativeReducer calculates the derivative of the aggregated points.
 type FloatDerivativeReducer struct {
 	interval      Interval
@@ -566,6 +725,392 @@ func (r *UnsignedMovingAverageReducer) Emit() []FloatPoint {
 	}
 }
 
+type ExponentialMovingAverageReducer struct {
+	ema        gota.EMA
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewExponentialMovingAverageReducer(period int, holdPeriod int, warmupType gota.WarmupType) *ExponentialMovingAverageReducer {
+	ema := gota.NewEMA(period, warmupType)
+	if holdPeriod == -1 {
+		holdPeriod = ema.WarmCount()
+	}
+	return &ExponentialMovingAverageReducer{
+		ema:        *ema,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+
+func (r *ExponentialMovingAverageReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *ExponentialMovingAverageReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *ExponentialMovingAverageReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *ExponentialMovingAverageReducer) aggregate(v float64, t int64) {
+	r.v = r.ema.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *ExponentialMovingAverageReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type DoubleExponentialMovingAverageReducer struct {
+	dema       gota.DEMA
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewDoubleExponentialMovingAverageReducer(period int, holdPeriod int, warmupType gota.WarmupType) *DoubleExponentialMovingAverageReducer {
+	dema := gota.NewDEMA(period, warmupType)
+	if holdPeriod == -1 {
+		holdPeriod = dema.WarmCount()
+	}
+	return &DoubleExponentialMovingAverageReducer{
+		dema:       *dema,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+
+func (r *DoubleExponentialMovingAverageReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *DoubleExponentialMovingAverageReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *DoubleExponentialMovingAverageReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *DoubleExponentialMovingAverageReducer) aggregate(v float64, t int64) {
+	r.v = r.dema.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *DoubleExponentialMovingAverageReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type TripleExponentialMovingAverageReducer struct {
+	tema       gota.TEMA
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewTripleExponentialMovingAverageReducer(period int, holdPeriod int, warmupType gota.WarmupType) *TripleExponentialMovingAverageReducer {
+	tema := gota.NewTEMA(period, warmupType)
+	if holdPeriod == -1 {
+		holdPeriod = tema.WarmCount()
+	}
+	return &TripleExponentialMovingAverageReducer{
+		tema:       *tema,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+
+func (r *TripleExponentialMovingAverageReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *TripleExponentialMovingAverageReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *TripleExponentialMovingAverageReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *TripleExponentialMovingAverageReducer) aggregate(v float64, t int64) {
+	r.v = r.tema.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *TripleExponentialMovingAverageReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type RelativeStrengthIndexReducer struct {
+	rsi        gota.RSI
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewRelativeStrengthIndexReducer(period int, holdPeriod int, warmupType gota.WarmupType) *RelativeStrengthIndexReducer {
+	rsi := gota.NewRSI(period, warmupType)
+	if holdPeriod == -1 {
+		holdPeriod = rsi.WarmCount()
+	}
+	return &RelativeStrengthIndexReducer{
+		rsi:        *rsi,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+func (r *RelativeStrengthIndexReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *RelativeStrengthIndexReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *RelativeStrengthIndexReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *RelativeStrengthIndexReducer) aggregate(v float64, t int64) {
+	r.v = r.rsi.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *RelativeStrengthIndexReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type TripleExponentialDerivativeReducer struct {
+	trix       gota.TRIX
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewTripleExponentialDerivativeReducer(period int, holdPeriod int, warmupType gota.WarmupType) *TripleExponentialDerivativeReducer {
+	trix := gota.NewTRIX(period, warmupType)
+	if holdPeriod == -1 {
+		holdPeriod = trix.WarmCount()
+	}
+	return &TripleExponentialDerivativeReducer{
+		trix:       *trix,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+func (r *TripleExponentialDerivativeReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *TripleExponentialDerivativeReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *TripleExponentialDerivativeReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *TripleExponentialDerivativeReducer) aggregate(v float64, t int64) {
+	r.v = r.trix.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *TripleExponentialDerivativeReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+	if math.IsInf(r.v, 0) {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type KaufmansEfficiencyRatioReducer struct {
+	ker        gota.KER
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewKaufmansEfficiencyRatioReducer(period int, holdPeriod int) *KaufmansEfficiencyRatioReducer {
+	ker := gota.NewKER(period)
+	if holdPeriod == -1 {
+		holdPeriod = ker.WarmCount()
+	}
+	return &KaufmansEfficiencyRatioReducer{
+		ker:        *ker,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+func (r *KaufmansEfficiencyRatioReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *KaufmansEfficiencyRatioReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *KaufmansEfficiencyRatioReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *KaufmansEfficiencyRatioReducer) aggregate(v float64, t int64) {
+	r.v = r.ker.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *KaufmansEfficiencyRatioReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+	if math.IsInf(r.v, 0) {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type KaufmansAdaptiveMovingAverageReducer struct {
+	kama       gota.KAMA
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewKaufmansAdaptiveMovingAverageReducer(period int, holdPeriod int) *KaufmansAdaptiveMovingAverageReducer {
+	kama := gota.NewKAMA(period)
+	if holdPeriod == -1 {
+		holdPeriod = kama.WarmCount()
+	}
+	return &KaufmansAdaptiveMovingAverageReducer{
+		kama:       *kama,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+func (r *KaufmansAdaptiveMovingAverageReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *KaufmansAdaptiveMovingAverageReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *KaufmansAdaptiveMovingAverageReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *KaufmansAdaptiveMovingAverageReducer) aggregate(v float64, t int64) {
+	r.v = r.kama.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *KaufmansAdaptiveMovingAverageReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+	if math.IsInf(r.v, 0) {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
+type ChandeMomentumOscillatorReducer struct {
+	cmo        gota.AlgSimple
+	holdPeriod uint32
+	count      uint32
+	v          float64
+	t          int64
+}
+
+func NewChandeMomentumOscillatorReducer(period int, holdPeriod int, warmupType gota.WarmupType) *ChandeMomentumOscillatorReducer {
+	var cmo gota.AlgSimple
+	if warmupType == gota.WarmupType(-1) {
+		cmo = gota.NewCMO(period)
+	} else {
+		cmo = gota.NewCMOS(period, warmupType)
+	}
+
+	if holdPeriod == -1 {
+		holdPeriod = cmo.WarmCount()
+	}
+	return &ChandeMomentumOscillatorReducer{
+		cmo:        cmo,
+		holdPeriod: uint32(holdPeriod),
+	}
+}
+func (r *ChandeMomentumOscillatorReducer) AggregateFloat(p *FloatPoint) {
+	r.aggregate(p.Value, p.Time)
+}
+func (r *ChandeMomentumOscillatorReducer) AggregateInteger(p *IntegerPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *ChandeMomentumOscillatorReducer) AggregateUnsigned(p *UnsignedPoint) {
+	r.aggregate(float64(p.Value), p.Time)
+}
+func (r *ChandeMomentumOscillatorReducer) aggregate(v float64, t int64) {
+	r.v = r.cmo.Add(v)
+	r.t = t
+	r.count++
+}
+func (r *ChandeMomentumOscillatorReducer) Emit() []FloatPoint {
+	if r.count <= r.holdPeriod {
+		return nil
+	}
+
+	return []FloatPoint{
+		{
+			Value:      r.v,
+			Time:       r.t,
+			Aggregated: r.count,
+		},
+	}
+}
+
 // FloatCumulativeSumReducer cumulates the values from each point.
 type FloatCumulativeSumReducer struct {
 	curr FloatPoint
