@@ -305,6 +305,29 @@ func NewFilterIterator(input Iterator, cond influxql.Expr, opt IteratorOptions)
 	}
 }
 
+// NewTagSubsetIterator will strip each of the points to a subset of the tag key values
+// for each point it processes.
+func NewTagSubsetIterator(input Iterator, opt IteratorOptions) Iterator {
+	if input == nil {
+		return nil
+	}
+
+	switch input := input.(type) {
+	case FloatIterator:
+		return newFloatTagSubsetIterator(input, opt)
+	case IntegerIterator:
+		return newIntegerTagSubsetIterator(input, opt)
+	case UnsignedIterator:
+		return newUnsignedTagSubsetIterator(input, opt)
+	case StringIterator:
+		return newStringTagSubsetIterator(input, opt)
+	case BooleanIterator:
+		return newBooleanTagSubsetIterator(input, opt)
+	default:
+		panic(fmt.Sprintf("unsupported tag subset iterator type: %T", input))
+	}
+}
+
 // NewDedupeIterator returns an iterator that only outputs unique points.
 // This iterator maintains a serialized copy of each row so it is inefficient
 // to use on large datasets. It is intended for small datasets such as meta queries.
