@@ -12,10 +12,10 @@ import (
 	"time"
 
 	"github.com/influxdata/influxdb/internal"
+	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/services/meta"
 	"github.com/influxdata/influxdb/toml"
-	"github.com/uber-go/zap"
 )
 
 func TestService_OpenClose(t *testing.T) {
@@ -87,10 +87,7 @@ func TestService_Open_TypesDBDir(t *testing.T) {
 	}
 
 	if testing.Verbose() {
-		s.Service.WithLogger(zap.New(
-			zap.NewTextEncoder(),
-			zap.Output(os.Stderr),
-		))
+		s.Service.WithLogger(logger.New(os.Stderr))
 	}
 
 	s.MetaClient.CreateDatabaseFn = func(name string) (*meta.DatabaseInfo, error) {
@@ -424,10 +421,7 @@ func NewTestService(batchSize int, batchDuration time.Duration, parseOpt string)
 	}
 
 	if testing.Verbose() {
-		s.Service.WithLogger(zap.New(
-			zap.NewTextEncoder(),
-			zap.Output(os.Stderr),
-		))
+		s.Service.WithLogger(logger.New(os.Stderr))
 	}
 
 	return s
