@@ -11,27 +11,32 @@ import (
 	"net/http"
 	"net/http/httptest"
 	"net/url"
+	"os"
 	"reflect"
 	"strings"
+	"sync/atomic"
 	"testing"
 	"time"
 
 	"github.com/dgrijalva/jwt-go"
 	"github.com/gogo/protobuf/proto"
 	"github.com/golang/snappy"
+	"github.com/google/go-cmp/cmp"
 	"github.com/influxdata/influxdb/internal"
+	"github.com/influxdata/influxdb/logger"
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/prometheus/remote"
 	"github.com/influxdata/influxdb/query"
 	"github.com/influxdata/influxdb/services/httpd"
 	"github.com/influxdata/influxdb/services/meta"
+	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxql"
 )
 
 // Ensure the handler returns results from a query (including nil results).
 func TestHandler_Query(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		if stmt.String() != `SELECT * FROM bar` {
 			t.Fatalf("unexpected query: %s", stmt.String())
 		} else if ctx.Database != `foo` {
@@ -54,7 +59,7 @@ func TestHandler_Query(t *testing.T) {
 // Ensure the handler returns results from a query passed as a file.
 func TestHandler_Query_File(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		if stmt.String() != `SELECT * FROM bar` {
 			t.Fatalf("unexpected query: %s", stmt.String())
 		} else if ctx.Database != `foo` {
@@ -123,7 +128,7 @@ func TestHandler_Query_Auth(t *testing.T) {
 	}
 
 	// Set mock statement executor for handler to use.
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		if stmt.String() != `SELECT * FROM bar` {
 			t.Fatalf("unexpected query: %s", stmt.String())
 		} else if ctx.Database != `foo` {
@@ -238,7 +243,7 @@ func TestHandler_Query_Auth(t *testing.T) {
 // Ensure the handler returns results from a query (including nil results).
 func TestHandler_QueryRegex(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		if stmt.String() != `SELECT * FROM test WHERE url =~ /http\:\/\/www.akamai\.com/` {
 			t.Fatalf("unexpected query: %s", stmt.String())
 		} else if ctx.Database != `test` {
@@ -255,7 +260,7 @@ func TestHandler_QueryRegex(t *testing.T) {
 // Ensure the handler merges results from the same statement.
 func TestHandler_Query_MergeResults(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		ctx.Results <- &query.Result{StatementID: 1, Series: models.Rows([]*models.Row{{Name: "series0"}})}
 		ctx.Results <- &query.Result{StatementID: 1, Series: models.Rows([]*models.Row{{Name: "series1"}})}
 		return nil
@@ -273,7 +278,7 @@ func TestHandler_Query_MergeResults(t *testing.T) {
 // Ensure the handler merges results from the same statement.
 func TestHandler_Query_MergeEmptyResults(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		ctx.Results <- &query.Result{StatementID: 1, Series: models.Rows{}}
 		ctx.Results <- &query.Result{StatementID: 1, Series: models.Rows([]*models.Row{{Name: "series1"}})}
 		return nil
@@ -291,7 +296,7 @@ func TestHandler_Query_MergeEmptyResults(t *testing.T) {
 // Ensure the handler can parse chunked and chunk size query parameters.
 func TestHandler_Query_Chunked(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		if ctx.ChunkSize != 2 {
 			t.Fatalf("unexpected chunk size: %d", ctx.ChunkSize)
 		}
@@ -315,7 +320,7 @@ func TestHandler_Query_Chunked(t *testing.T) {
 func TestHandler_Query_Async(t *testing.T) {
 	done := make(chan struct{})
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		if stmt.String() != `SELECT * FROM bar` {
 			t.Fatalf("unexpected query: %s", stmt.String())
 		} else if ctx.Database != `foo` {
@@ -449,7 +454,7 @@ func TestHandler_Query_ErrAuthorize(t *testing.T) {
 // Ensure the handler returns a status 200 if an error is returned in the result.
 func TestHandler_Query_ErrResult(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		return errors.New("measurement not found")
 	}
 
@@ -470,9 +475,9 @@ func TestHandler_Query_CloseNotify(t *testing.T) {
 
 	interrupted := make(chan struct{})
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		select {
-		case <-ctx.InterruptCh:
+		case <-ctx.Done():
 		case <-done:
 		}
 		close(interrupted)
@@ -574,7 +579,7 @@ func TestHandler_PromWrite(t *testing.T) {
 		if err != nil {
 			t.Fatal(err.Error())
 		}
-		expFields := models.Fields{"f64": 1.2}
+		expFields := models.Fields{"value": 1.2}
 		if !reflect.DeepEqual(fields, expFields) {
 			t.Fatalf("fields don't match\n\texp: %v\n\tgot: %v", expFields, fields)
 		}
@@ -597,10 +602,11 @@ func TestHandler_PromRead(t *testing.T) {
 	req := &remote.ReadRequest{
 		Queries: []*remote.Query{{
 			Matchers: []*remote.LabelMatcher{
-				{Type: remote.MatchType_EQUAL, Name: "eq", Value: "a"},
-				{Type: remote.MatchType_NOT_EQUAL, Name: "neq", Value: "b"},
-				{Type: remote.MatchType_REGEX_MATCH, Name: "regex", Value: "c"},
-				{Type: remote.MatchType_REGEX_NO_MATCH, Name: "neqregex", Value: "d"},
+				{
+					Type:  remote.MatchType_EQUAL,
+					Name:  "__name__",
+					Value: "value",
+				},
 			},
 			StartTimestampMs: 1,
 			EndTimestampMs:   2,
@@ -612,31 +618,116 @@ func TestHandler_PromRead(t *testing.T) {
 	}
 	compressed := snappy.Encode(nil, data)
 	b := bytes.NewReader(compressed)
-
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
-		if stmt.String() != `SELECT f64 FROM foo.._ WHERE eq = 'a' AND neq != 'b' AND regex =~ /c/ AND neqregex !~ /d/ AND time >= '1970-01-01T00:00:00.001Z' AND time <= '1970-01-01T00:00:00.002Z' GROUP BY *` {
-			t.Fatalf("unexpected query: %s", stmt.String())
-		} else if ctx.Database != `foo` {
-			t.Fatalf("unexpected db: %s", ctx.Database)
-		}
-		row := &models.Row{
-			Name:    "_",
-			Tags:    map[string]string{"foo": "bar"},
-			Columns: []string{"time", "f64"},
-			Values:  [][]interface{}{{time.Unix(23, 0), 1.2}},
-		}
-		ctx.Results <- &query.Result{StatementID: 1, Series: models.Rows([]*models.Row{row})}
-		return nil
-	}
-
 	w := httptest.NewRecorder()
 
-	h.ServeHTTP(w, MustNewJSONRequest("POST", "/api/v1/prom/read?db=foo", b))
+	// Number of results in the result set
+	var i int64
+	h.Store.ResultSet.NextFn = func() bool {
+		i++
+		return i <= 2
+	}
+
+	// data for each cursor.
+	h.Store.ResultSet.CursorFn = func() tsdb.Cursor {
+		cursor := internal.NewFloatBatchCursorMock()
+
+		var i int64
+		cursor.NextFn = func() ([]int64, []float64) {
+			i++
+			ts := []int64{22000000 * i, 10000000000 * i}
+			vs := []float64{2.3, 2992.33}
+			if i > 2 {
+				ts, vs = nil, nil
+			}
+			return ts, vs
+		}
+
+		return cursor
+	}
+
+	// Tags for each cursor.
+	h.Store.ResultSet.TagsFn = func() models.Tags {
+		return models.NewTags(map[string]string{
+			"host":         fmt.Sprintf("server-%d", i),
+			"_measurement": "mem",
+		})
+	}
+
+	h.ServeHTTP(w, MustNewRequest("POST", "/api/v1/prom/read?db=foo&rp=bar", b))
 	if w.Code != http.StatusOK {
 		t.Fatalf("unexpected status: %d", w.Code)
 	}
 
+	reqBuf, err := snappy.Decode(nil, w.Body.Bytes())
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	var resp remote.ReadResponse
+	if err := proto.Unmarshal(reqBuf, &resp); err != nil {
+		t.Fatal(err)
+	}
+
+	expResults := []*remote.QueryResult{
+		{
+			Timeseries: []*remote.TimeSeries{
+				{
+					Labels: []*remote.LabelPair{
+						{Name: "host", Value: "server-1"},
+					},
+					Samples: []*remote.Sample{
+						{TimestampMs: 22, Value: 2.3},
+						{TimestampMs: 10000, Value: 2992.33},
+						{TimestampMs: 44, Value: 2.3},
+						{TimestampMs: 20000, Value: 2992.33},
+					},
+				},
+				{
+					Labels: []*remote.LabelPair{
+						{Name: "host", Value: "server-2"},
+					},
+					Samples: []*remote.Sample{
+						{TimestampMs: 22, Value: 2.3},
+						{TimestampMs: 10000, Value: 2992.33},
+						{TimestampMs: 44, Value: 2.3},
+						{TimestampMs: 20000, Value: 2992.33},
+					},
+				},
+			},
+		},
+	}
+
+	if !reflect.DeepEqual(resp.Results, expResults) {
+		t.Fatalf("Results differ:\n%v", cmp.Diff(resp.Results, expResults))
+	}
+}
+
+func TestHandler_PromRead_NoResults(t *testing.T) {
+	req := &remote.ReadRequest{Queries: []*remote.Query{&remote.Query{
+		Matchers: []*remote.LabelMatcher{
+			{
+				Type:  remote.MatchType_EQUAL,
+				Name:  "__name__",
+				Value: "value",
+			},
+		},
+		StartTimestampMs: 0,
+		EndTimestampMs:   models.MaxNanoTime / int64(time.Millisecond),
+	}}}
+	data, err := proto.Marshal(req)
+	if err != nil {
+		t.Fatal("couldn't marshal prometheus request")
+	}
+	compressed := snappy.Encode(nil, data)
+	h := NewHandler(false)
+	w := httptest.NewRecorder()
+
+	b := bytes.NewReader(compressed)
+	h.ServeHTTP(w, MustNewJSONRequest("POST", "/api/v1/prom/read?db=foo", b))
+	if w.Code != http.StatusOK {
+		t.Fatalf("unexpected status: %d", w.Code)
+	}
 	reqBuf, err := snappy.Decode(nil, w.Body.Bytes())
 	if err != nil {
 		t.Fatal(err.Error())
@@ -646,17 +737,66 @@ func TestHandler_PromRead(t *testing.T) {
 	if err := proto.Unmarshal(reqBuf, &resp); err != nil {
 		t.Fatal(err.Error())
 	}
+}
 
-	expLabels := []*remote.LabelPair{{Name: "foo", Value: "bar"}}
-	expSamples := []*remote.Sample{{TimestampMs: 23000, Value: 1.2}}
-
-	ts := resp.Results[0].Timeseries[0]
-
-	if !reflect.DeepEqual(expLabels, ts.Labels) {
-		t.Fatalf("unexpected labels\n\texp: %v\n\tgot: %v", expLabels, ts.Labels)
+func TestHandler_PromRead_UnsupportedCursors(t *testing.T) {
+	req := &remote.ReadRequest{Queries: []*remote.Query{&remote.Query{
+		Matchers: []*remote.LabelMatcher{
+			{
+				Type:  remote.MatchType_EQUAL,
+				Name:  "__name__",
+				Value: "value",
+			},
+		},
+		StartTimestampMs: 0,
+		EndTimestampMs:   models.MaxNanoTime / int64(time.Millisecond),
+	}}}
+	data, err := proto.Marshal(req)
+	if err != nil {
+		t.Fatal("couldn't marshal prometheus request")
 	}
-	if !reflect.DeepEqual(expSamples, ts.Samples) {
-		t.Fatalf("unexpectd samples\n\texp: %v\n\tgot: %v", expSamples, ts.Samples)
+	compressed := snappy.Encode(nil, data)
+
+	unsupported := []tsdb.Cursor{
+		internal.NewIntegerBatchCursorMock(),
+		internal.NewBooleanBatchCursorMock(),
+		internal.NewUnsignedBatchCursorMock(),
+		internal.NewStringBatchCursorMock(),
+	}
+
+	for _, cursor := range unsupported {
+		h := NewHandler(false)
+		w := httptest.NewRecorder()
+		var lb bytes.Buffer
+		h.Logger = logger.New(&lb)
+
+		more := true
+		h.Store.ResultSet.NextFn = func() bool { defer func() { more = false }(); return more }
+
+		// Set the cursor type that will be returned while iterating over
+		// the mock store.
+		h.Store.ResultSet.CursorFn = func() tsdb.Cursor {
+			return cursor
+		}
+
+		b := bytes.NewReader(compressed)
+		h.ServeHTTP(w, MustNewJSONRequest("POST", "/api/v1/prom/read?db=foo", b))
+		if w.Code != http.StatusOK {
+			t.Fatalf("unexpected status: %d", w.Code)
+		}
+		reqBuf, err := snappy.Decode(nil, w.Body.Bytes())
+		if err != nil {
+			t.Fatal(err.Error())
+		}
+
+		var resp remote.ReadResponse
+		if err := proto.Unmarshal(reqBuf, &resp); err != nil {
+			t.Fatal(err.Error())
+		}
+
+		if !strings.Contains(lb.String(), "cursor_type=") {
+			t.Fatalf("got log message %q, expected to contain \"cursor_type\"", lb.String())
+		}
 	}
 }
 
@@ -678,7 +818,7 @@ func TestHandler_Ping(t *testing.T) {
 // Ensure the handler returns the version correctly from the different endpoints.
 func TestHandler_Version(t *testing.T) {
 	h := NewHandler(false)
-	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx query.ExecutionContext) error {
+	h.StatementExecutor.ExecuteStatementFn = func(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 		return nil
 	}
 	tests := []struct {
@@ -769,6 +909,32 @@ func TestHandler_Write_EntityTooLarge_ContentLength(t *testing.T) {
 	}
 }
 
+func TestHandler_Write_SuppressLog(t *testing.T) {
+	var buf bytes.Buffer
+	c := httpd.NewConfig()
+	c.SuppressWriteLog = true
+	h := NewHandlerWithConfig(c)
+	h.CLFLogger = log.New(&buf, "", log.LstdFlags)
+	h.MetaClient.DatabaseFn = func(name string) *meta.DatabaseInfo {
+		return &meta.DatabaseInfo{}
+	}
+	h.PointsWriter.WritePointsFn = func(database, retentionPolicy string, consistencyLevel models.ConsistencyLevel, user meta.User, points []models.Point) error {
+		return nil
+	}
+
+	b := strings.NewReader("cpu,host=server01 value=2\n")
+	w := httptest.NewRecorder()
+	h.ServeHTTP(w, MustNewRequest("POST", "/write?db=foo", b))
+	if w.Code != http.StatusNoContent {
+		t.Fatalf("unexpected status: %d", w.Code)
+	}
+
+	// If the log has anything in it, this failed.
+	if buf.Len() > 0 {
+		t.Fatalf("expected no bytes to be written to the log, got %d", buf.Len())
+	}
+}
+
 // onlyReader implements io.Reader only to ensure Request.ContentLength is not set
 type onlyReader struct {
 	r io.Reader
@@ -900,6 +1066,101 @@ func TestHandler_XRequestId(t *testing.T) {
 	}
 }
 
+func TestThrottler_Handler(t *testing.T) {
+	t.Run("OK", func(t *testing.T) {
+		throttler := httpd.NewThrottler(2, 98)
+
+		// Send the total number of concurrent requests to the channel.
+		var concurrentN int32
+		concurrentCh := make(chan int)
+
+		h := throttler.Handler(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			atomic.AddInt32(&concurrentN, 1)
+			concurrentCh <- int(atomic.LoadInt32(&concurrentN))
+			time.Sleep(1 * time.Millisecond)
+			atomic.AddInt32(&concurrentN, -1)
+		}))
+
+		// Execute requests concurrently.
+		const n = 100
+		for i := 0; i < n; i++ {
+			go func() { h.ServeHTTP(nil, nil) }()
+		}
+
+		// Read the number of concurrent requests for every execution.
+		for i := 0; i < n; i++ {
+			if v := <-concurrentCh; v > 2 {
+				t.Fatalf("concurrent requests exceed maximum: %d", v)
+			}
+		}
+	})
+
+	t.Run("ErrTimeout", func(t *testing.T) {
+		throttler := httpd.NewThrottler(2, 1)
+		throttler.EnqueueTimeout = 1 * time.Millisecond
+
+		begin, end := make(chan struct{}), make(chan struct{})
+		h := throttler.Handler(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			begin <- struct{}{}
+			end <- struct{}{}
+		}))
+
+		// First two requests should execute immediately.
+		go func() { h.ServeHTTP(nil, nil) }()
+		go func() { h.ServeHTTP(nil, nil) }()
+
+		<-begin
+		<-begin
+
+		// Third request should be enqueued but timeout.
+		w := httptest.NewRecorder()
+		h.ServeHTTP(w, nil)
+		if w.Code != http.StatusServiceUnavailable {
+			t.Fatalf("unexpected status code: %d", w.Code)
+		} else if body := w.Body.String(); body != "request throttled, exceeds timeout\n" {
+			t.Fatalf("unexpected response body: %q", body)
+		}
+
+		// Allow 2 existing requests to complete.
+		<-end
+		<-end
+	})
+
+	t.Run("ErrFull", func(t *testing.T) {
+		delay := 100 * time.Millisecond
+		if os.Getenv("CI") != "" {
+			delay = 2 * time.Second
+		}
+
+		throttler := httpd.NewThrottler(2, 1)
+
+		resp := make(chan struct{})
+		h := throttler.Handler(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+			resp <- struct{}{}
+		}))
+
+		// First two requests should execute immediately and third should be queued.
+		go func() { h.ServeHTTP(nil, nil) }()
+		go func() { h.ServeHTTP(nil, nil) }()
+		go func() { h.ServeHTTP(nil, nil) }()
+		time.Sleep(delay)
+
+		// Fourth request should fail when trying to enqueue.
+		w := httptest.NewRecorder()
+		h.ServeHTTP(w, nil)
+		if w.Code != http.StatusServiceUnavailable {
+			t.Fatalf("unexpected status code: %d", w.Code)
+		} else if body := w.Body.String(); body != "request throttled, queue full\n" {
+			t.Fatalf("unexpected response body: %q", body)
+		}
+
+		// Allow 3 existing requests to complete.
+		<-resp
+		<-resp
+		<-resp
+	})
+}
+
 // NewHandler represents a test wrapper for httpd.Handler.
 type Handler struct {
 	*httpd.Handler
@@ -907,6 +1168,7 @@ type Handler struct {
 	StatementExecutor HandlerStatementExecutor
 	QueryAuthorizer   HandlerQueryAuthorizer
 	PointsWriter      HandlerPointsWriter
+	Store             *internal.StorageStoreMock
 }
 
 // NewHandler returns a new instance of Handler.
@@ -914,29 +1176,41 @@ func NewHandler(requireAuthentication bool) *Handler {
 	config := httpd.NewConfig()
 	config.AuthEnabled = requireAuthentication
 	config.SharedSecret = "super secret key"
+	return NewHandlerWithConfig(config)
+}
 
+func NewHandlerWithConfig(config httpd.Config) *Handler {
 	h := &Handler{
 		Handler: httpd.NewHandler(config),
 	}
 
 	h.MetaClient = &internal.MetaClientMock{}
+	h.Store = internal.NewStorageStoreMock()
 
 	h.Handler.MetaClient = h.MetaClient
-	h.Handler.QueryExecutor = query.NewQueryExecutor()
+	h.Handler.Store = h.Store
+	h.Handler.QueryExecutor = query.NewExecutor()
 	h.Handler.QueryExecutor.StatementExecutor = &h.StatementExecutor
 	h.Handler.QueryAuthorizer = &h.QueryAuthorizer
 	h.Handler.PointsWriter = &h.PointsWriter
 	h.Handler.Version = "0.0.0"
 	h.Handler.BuildType = "OSS"
+
+	if testing.Verbose() {
+		l := logger.New(os.Stdout)
+		h.Handler.Logger = l
+		h.Handler.Store.WithLogger(l)
+	}
+
 	return h
 }
 
 // HandlerStatementExecutor is a mock implementation of Handler.StatementExecutor.
 type HandlerStatementExecutor struct {
-	ExecuteStatementFn func(stmt influxql.Statement, ctx query.ExecutionContext) error
+	ExecuteStatementFn func(stmt influxql.Statement, ctx *query.ExecutionContext) error
 }
 
-func (e *HandlerStatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx query.ExecutionContext) error {
+func (e *HandlerStatementExecutor) ExecuteStatement(stmt influxql.Statement, ctx *query.ExecutionContext) error {
 	return e.ExecuteStatementFn(stmt, ctx)
 }
 
