@@ -389,7 +389,7 @@ type BasicQuery struct {
 
 // QueryGenerate returns a Query channel
 func (q *BasicQuery) QueryGenerate(now func() time.Time) (<-chan Query, error) {
-	c := make(chan Query, 0)
+	c := make(chan Query)
 
 	go func(chan Query) {
 		defer close(c)
@@ -406,7 +406,6 @@ func (q *BasicQuery) QueryGenerate(now func() time.Time) (<-chan Query, error) {
 // SetTime sets the internal state of time
 func (q *BasicQuery) SetTime(t time.Time) {
 	q.time = t
-	return
 }
 
 // BasicQueryClient implements the QueryClient interface
@@ -513,11 +512,7 @@ func resetDB(c client.Client, database string) error {
 	_, err = c.Query(client.Query{
 		Command: fmt.Sprintf("CREATE DATABASE %s", database),
 	})
-	if err != nil {
-		return err
-	}
-
-	return nil
+	return err
 }
 
 // BasicProvisioner implements the Provisioner
@@ -562,7 +557,7 @@ func NewBroadcastChannel() *BroadcastChannel {
 }
 
 func (b *BroadcastChannel) Register(fn responseHandler) {
-	ch := make(chan response, 0)
+	ch := make(chan response)
 
 	b.chs = append(b.chs, ch)
 
