@@ -7,6 +7,7 @@ import (
 	"io"
 	"os"
 	"regexp"
+	"runtime"
 	"sort"
 	"time"
 
@@ -48,13 +49,14 @@ type Engine interface {
 	Digest() (io.ReadCloser, int64, error)
 
 	CreateIterator(ctx context.Context, measurement string, opt query.IteratorOptions) (query.Iterator, error)
-	CreateCursor(ctx context.Context, r *CursorRequest) (Cursor, error)
+	CreateCursorIterator(ctx context.Context) (CursorIterator, error)
 	IteratorCost(measurement string, opt query.IteratorOptions) (query.IteratorCost, error)
 	WritePoints(points []models.Point) error
 
 	CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error
 	CreateSeriesListIfNotExists(keys, names [][]byte, tags []models.Tags) error
 	DeleteSeriesRange(itr SeriesIterator, min, max int64) error
+	DeleteSeriesRangeWithPredicate(itr SeriesIterator, predicate func(name []byte, tags models.Tags) (int64, int64, bool)) error
 
 	MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error)
 	SeriesSketches() (estimator.Sketch, estimator.Sketch, error)
@@ -96,7 +98,7 @@ const (
 )
 
 // NewEngineFunc creates a new engine.
-type NewEngineFunc func(id uint64, i Index, database, path string, walPath string, sfile *SeriesFile, options EngineOptions) Engine
+type NewEngineFunc func(id uint64, i Index, path string, walPath string, sfile *SeriesFile, options EngineOptions) Engine
 
 // newEngineFuncs is a lookup of engine constructors by name.
 var newEngineFuncs = make(map[string]NewEngineFunc)
@@ -121,10 +123,14 @@ func RegisteredEngines() []string {
 
 // NewEngine returns an instance of an engine based on its format.
 // If the path does not exist then the DefaultFormat is used.
-func NewEngine(id uint64, i Index, database, path string, walPath string, sfile *SeriesFile, options EngineOptions) (Engine, error) {
+func NewEngine(id uint64, i Index, path string, walPath string, sfile *SeriesFile, options EngineOptions) (Engine, error) {
 	// Create a new engine
 	if _, err := os.Stat(path); os.IsNotExist(err) {
-		return newEngineFuncs[options.EngineVersion](id, i, database, path, walPath, sfile, options), nil
+		engine := newEngineFuncs[options.EngineVersion](id, i, path, walPath, sfile, options)
+		if options.OnNewEngine != nil {
+			options.OnNewEngine(engine)
+		}
+		return engine, nil
 	}
 
 	// If it's a dir then it's a tsm1 engine
@@ -143,7 +149,11 @@ func NewEngine(id uint64, i Index, database, path string, walPath string, sfile
 		return nil, fmt.Errorf("invalid engine format: %q", format)
 	}
 
-	return fn(id, i, database, path, walPath, sfile, options), nil
+	engine := fn(id, i, path, walPath, sfile, options)
+	if options.OnNewEngine != nil {
+		options.OnNewEngine(engine)
+	}
+	return engine, nil
 }
 
 // EngineOptions represents the options used to initialize the engine.
@@ -153,21 +163,59 @@ type EngineOptions struct {
 	ShardID       uint64
 	InmemIndex    interface{} // shared in-memory index
 
+	// Limits the concurrent number of TSM files that can be loaded at once.
+	OpenLimiter limiter.Fixed
+
+	CompactionPlannerCreator    CompactionPlannerCreator
 	CompactionLimiter           limiter.Fixed
 	CompactionThroughputLimiter limiter.Rate
+	WALEnabled                  bool
+	MonitorDisabled             bool
 
-	Config       Config
-	SeriesIDSets SeriesIDSets
+	// DatabaseFilter is a predicate controlling which databases may be opened.
+	// If no function is set, all databases will be opened.
+	DatabaseFilter func(database string) bool
+
+	// RetentionPolicyFilter is a predicate controlling which combination of database and retention policy may be opened.
+	// nil will allow all combinations to pass.
+	RetentionPolicyFilter func(database, rp string) bool
+
+	// ShardFilter is a predicate controlling which combination of database, retention policy and shard group may be opened.
+	// nil will allow all combinations to pass.
+	ShardFilter func(database, rp string, id uint64) bool
+
+	Config         Config
+	SeriesIDSets   SeriesIDSets
+	FieldValidator FieldValidator
+
+	OnNewEngine func(Engine)
+
+	FileStoreObserver FileStoreObserver
 }
 
-// NewEngineOptions returns the default options.
+// NewEngineOptions constructs an EngineOptions object with safe default values.
+// This should only be used in tests; production environments should read from a config file.
 func NewEngineOptions() EngineOptions {
 	return EngineOptions{
 		EngineVersion: DefaultEngine,
 		IndexVersion:  DefaultIndex,
 		Config:        NewConfig(),
+		WALEnabled:    true,
+		OpenLimiter:   limiter.NewFixed(runtime.GOMAXPROCS(0)),
 	}
 }
 
 // NewInmemIndex returns a new "inmem" index type.
 var NewInmemIndex func(name string, sfile *SeriesFile) (interface{}, error)
+
+type CompactionPlannerCreator func(cfg Config) interface{}
+
+// FileStoreObserver is passed notifications before the file store adds or deletes files. In this way, it can
+// be sure to observe every file that is added or removed even in the presence of process death.
+type FileStoreObserver interface {
+	// FileFinishing is called before a file is renamed to it's final name.
+	FileFinishing(path string) error
+
+	// FileUnlinking is called before a file is unlinked.
+	FileUnlinking(path string) error
+}
