@@ -23,23 +23,22 @@ func TestCompactor_Snapshot(t *testing.T) {
 	v3 := tsm1.NewValue(2, float64(2))
 
 	points1 := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v1},
-		"cpu,host=B#!~#value": []tsm1.Value{v2, v3},
+		"cpu,host=A#!~#value": {v1},
+		"cpu,host=B#!~#value": {v2, v3},
 	}
 
-	c := tsm1.NewCache(0, "")
+	c := tsm1.NewCache(0)
 	for k, v := range points1 {
 		if err := c.Write([]byte(k), v); err != nil {
 			t.Fatalf("failed to write key foo to cache: %s", err.Error())
 		}
 	}
 
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: &fakeFileStore{},
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = &fakeFileStore{}
 
-	files, err := compactor.WriteSnapshot(c)
+	files, err := compactor.WriteSnapshot(c, tsm1.DefaultFormatFileName)
 	if err == nil {
 		t.Fatalf("expected error writing snapshot: %v", err)
 	}
@@ -50,7 +49,7 @@ func TestCompactor_Snapshot(t *testing.T) {
 
 	compactor.Open()
 
-	files, err = compactor.WriteSnapshot(c)
+	files, err = compactor.WriteSnapshot(c, tsm1.DefaultFormatFileName)
 	if err != nil {
 		t.Fatalf("unexpected error writing snapshot: %v", err)
 	}
@@ -97,32 +96,31 @@ func TestCompactor_CompactFull(t *testing.T) {
 	// write 3 TSM files with different data and one new point
 	a1 := tsm1.NewValue(1, 1.1)
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1},
+		"cpu,host=A#!~#value": {a1},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
 	a2 := tsm1.NewValue(2, 1.2)
 	b1 := tsm1.NewValue(1, 2.1)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a2},
-		"cpu,host=B#!~#value": []tsm1.Value{b1},
+		"cpu,host=A#!~#value": {a2},
+		"cpu,host=B#!~#value": {b1},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
 	a3 := tsm1.NewValue(1, 1.3)
 	c1 := tsm1.NewValue(1, 3.1)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a3},
-		"cpu,host=C#!~#value": []tsm1.Value{c1},
+		"cpu,host=A#!~#value": {a3},
+		"cpu,host=C#!~#value": {c1},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
 
 	files, err := compactor.CompactFull([]string{f1, f2, f3})
 	if err == nil {
@@ -144,13 +142,13 @@ func TestCompactor_CompactFull(t *testing.T) {
 		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
 	}
 
-	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	expGen, expSeq, err := tsm1.ParseFileName(f3)
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
 	expSeq = expSeq + 1
 
-	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	gotGen, gotSeq, err := tsm1.ParseFileName(files[0])
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
@@ -205,7 +203,7 @@ func TestCompactor_Compact_OverlappingBlocks(t *testing.T) {
 	a3 := tsm1.NewValue(7, 1.1)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3},
+		"cpu,host=A#!~#value": {a1, a2, a3},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
@@ -214,17 +212,16 @@ func TestCompactor_Compact_OverlappingBlocks(t *testing.T) {
 	c3 := tsm1.NewValue(9, 1.2)
 
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{c1, c2, c3},
+		"cpu,host=A#!~#value": {c1, c2, c3},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
+	compactor.Size = 2
 
 	compactor.Open()
 
@@ -277,7 +274,7 @@ func TestCompactor_Compact_OverlappingBlocksMultiple(t *testing.T) {
 	a3 := tsm1.NewValue(7, 1.1)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3},
+		"cpu,host=A#!~#value": {a1, a2, a3},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
@@ -286,7 +283,7 @@ func TestCompactor_Compact_OverlappingBlocksMultiple(t *testing.T) {
 	b3 := tsm1.NewValue(6, 1.2)
 
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{b1, b2, b3},
+		"cpu,host=A#!~#value": {b1, b2, b3},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
@@ -295,17 +292,16 @@ func TestCompactor_Compact_OverlappingBlocksMultiple(t *testing.T) {
 	c3 := tsm1.NewValue(9, 1.2)
 
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{c1, c2, c3},
+		"cpu,host=A#!~#value": {c1, c2, c3},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
+	compactor.Size = 2
 
 	compactor.Open()
 
@@ -357,7 +353,7 @@ func TestCompactor_Compact_UnsortedBlocks(t *testing.T) {
 	a3 := tsm1.NewValue(6, 1.1)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3},
+		"cpu,host=A#!~#value": {a1, a2, a3},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
@@ -366,15 +362,14 @@ func TestCompactor_Compact_UnsortedBlocks(t *testing.T) {
 	b3 := tsm1.NewValue(3, 1.2)
 
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{b1, b2, b3},
+		"cpu,host=A#!~#value": {b1, b2, b3},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: &fakeFileStore{},
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = &fakeFileStore{}
+	compactor.Size = 2
 
 	compactor.Open()
 
@@ -425,7 +420,7 @@ func TestCompactor_Compact_UnsortedBlocksOverlapping(t *testing.T) {
 	a2 := tsm1.NewValue(2, 1.1)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
+		"cpu,host=A#!~#value": {a1, a2},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
@@ -433,7 +428,7 @@ func TestCompactor_Compact_UnsortedBlocksOverlapping(t *testing.T) {
 	b2 := tsm1.NewValue(4, 1.2)
 
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{b1, b2},
+		"cpu,host=A#!~#value": {b1, b2},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
@@ -441,15 +436,14 @@ func TestCompactor_Compact_UnsortedBlocksOverlapping(t *testing.T) {
 	c2 := tsm1.NewValue(2, 1.1)
 
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{c1, c2},
+		"cpu,host=A#!~#value": {c1, c2},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: &fakeFileStore{},
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = &fakeFileStore{}
+	compactor.Size = 2
 
 	compactor.Open()
 
@@ -500,29 +494,28 @@ func TestCompactor_CompactFull_SkipFullBlocks(t *testing.T) {
 	a1 := tsm1.NewValue(1, 1.1)
 	a2 := tsm1.NewValue(2, 1.2)
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
+		"cpu,host=A#!~#value": {a1, a2},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
 	a3 := tsm1.NewValue(3, 1.3)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a3},
+		"cpu,host=A#!~#value": {a3},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
 	a4 := tsm1.NewValue(4, 1.4)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a4},
+		"cpu,host=A#!~#value": {a4},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
+	compactor.Size = 2
 	compactor.Open()
 
 	files, err := compactor.CompactFull([]string{f1, f2, f3})
@@ -534,13 +527,13 @@ func TestCompactor_CompactFull_SkipFullBlocks(t *testing.T) {
 		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
 	}
 
-	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	expGen, expSeq, err := tsm1.ParseFileName(f3)
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
 	expSeq = expSeq + 1
 
-	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	gotGen, gotSeq, err := tsm1.ParseFileName(files[0])
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
@@ -596,13 +589,11 @@ func TestCompactor_CompactFull_TombstonedSkipBlock(t *testing.T) {
 	a1 := tsm1.NewValue(1, 1.1)
 	a2 := tsm1.NewValue(2, 1.2)
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
+		"cpu,host=A#!~#value": {a1, a2},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
-	ts := tsm1.Tombstoner{
-		Path: f1,
-	}
+	ts := tsm1.NewTombstoner(f1, nil)
 	ts.AddRange([][]byte{[]byte("cpu,host=A#!~#value")}, math.MinInt64, math.MaxInt64)
 
 	if err := ts.Flush(); err != nil {
@@ -611,23 +602,22 @@ func TestCompactor_CompactFull_TombstonedSkipBlock(t *testing.T) {
 
 	a3 := tsm1.NewValue(3, 1.3)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a3},
+		"cpu,host=A#!~#value": {a3},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
 	a4 := tsm1.NewValue(4, 1.4)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a4},
+		"cpu,host=A#!~#value": {a4},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
+	compactor.Size = 2
 	compactor.Open()
 
 	files, err := compactor.CompactFull([]string{f1, f2, f3})
@@ -639,13 +629,13 @@ func TestCompactor_CompactFull_TombstonedSkipBlock(t *testing.T) {
 		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
 	}
 
-	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	expGen, expSeq, err := tsm1.ParseFileName(f3)
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
 	expSeq = expSeq + 1
 
-	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	gotGen, gotSeq, err := tsm1.ParseFileName(files[0])
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
@@ -701,13 +691,11 @@ func TestCompactor_CompactFull_TombstonedPartialBlock(t *testing.T) {
 	a1 := tsm1.NewValue(1, 1.1)
 	a2 := tsm1.NewValue(2, 1.2)
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
+		"cpu,host=A#!~#value": {a1, a2},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
-	ts := tsm1.Tombstoner{
-		Path: f1,
-	}
+	ts := tsm1.NewTombstoner(f1, nil)
 	// a1 should remain after compaction
 	ts.AddRange([][]byte{[]byte("cpu,host=A#!~#value")}, 2, math.MaxInt64)
 
@@ -717,23 +705,22 @@ func TestCompactor_CompactFull_TombstonedPartialBlock(t *testing.T) {
 
 	a3 := tsm1.NewValue(3, 1.3)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a3},
+		"cpu,host=A#!~#value": {a3},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
 	a4 := tsm1.NewValue(4, 1.4)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a4},
+		"cpu,host=A#!~#value": {a4},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
+	compactor.Size = 2
 	compactor.Open()
 
 	files, err := compactor.CompactFull([]string{f1, f2, f3})
@@ -745,13 +732,13 @@ func TestCompactor_CompactFull_TombstonedPartialBlock(t *testing.T) {
 		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
 	}
 
-	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	expGen, expSeq, err := tsm1.ParseFileName(f3)
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
 	expSeq = expSeq + 1
 
-	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	gotGen, gotSeq, err := tsm1.ParseFileName(files[0])
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
@@ -811,13 +798,11 @@ func TestCompactor_CompactFull_TombstonedMultipleRanges(t *testing.T) {
 	a4 := tsm1.NewValue(4, 1.4)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3, a4},
+		"cpu,host=A#!~#value": {a1, a2, a3, a4},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
-	ts := tsm1.Tombstoner{
-		Path: f1,
-	}
+	ts := tsm1.NewTombstoner(f1, nil)
 	// a1, a3 should remain after compaction
 	ts.AddRange([][]byte{[]byte("cpu,host=A#!~#value")}, 2, 2)
 	ts.AddRange([][]byte{[]byte("cpu,host=A#!~#value")}, 4, 4)
@@ -828,23 +813,22 @@ func TestCompactor_CompactFull_TombstonedMultipleRanges(t *testing.T) {
 
 	a5 := tsm1.NewValue(5, 1.5)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a5},
+		"cpu,host=A#!~#value": {a5},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
 	a6 := tsm1.NewValue(6, 1.6)
 	writes = map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{a6},
+		"cpu,host=A#!~#value": {a6},
 	}
 	f3 := MustWriteTSM(dir, 3, writes)
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-		Size:      2,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
+	compactor.Size = 2
 	compactor.Open()
 
 	files, err := compactor.CompactFull([]string{f1, f2, f3})
@@ -856,13 +840,13 @@ func TestCompactor_CompactFull_TombstonedMultipleRanges(t *testing.T) {
 		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
 	}
 
-	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	expGen, expSeq, err := tsm1.ParseFileName(f3)
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
 	expSeq = expSeq + 1
 
-	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	gotGen, gotSeq, err := tsm1.ParseFileName(files[0])
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
@@ -957,10 +941,9 @@ func TestCompactor_CompactFull_MaxKeys(t *testing.T) {
 
 	fs := &fakeFileStore{}
 	defer fs.Close()
-	compactor := &tsm1.Compactor{
-		Dir:       dir,
-		FileStore: fs,
-	}
+	compactor := tsm1.NewCompactor()
+	compactor.Dir = dir
+	compactor.FileStore = fs
 	compactor.Open()
 
 	// Compact both files, should get 2 files back
@@ -973,13 +956,13 @@ func TestCompactor_CompactFull_MaxKeys(t *testing.T) {
 		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
 	}
 
-	expGen, expSeq, err := tsm1.ParseTSMFileName(f2Name)
+	expGen, expSeq, err := tsm1.ParseFileName(f2Name)
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
 	expSeq = expSeq + 1
 
-	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	gotGen, gotSeq, err := tsm1.ParseFileName(files[0])
 	if err != nil {
 		t.Fatalf("unexpected error parsing file name: %v", err)
 	}
@@ -1000,7 +983,7 @@ func TestTSMKeyIterator_Single(t *testing.T) {
 
 	v1 := tsm1.NewValue(1, 1.1)
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v1},
+		"cpu,host=A#!~#value": {v1},
 	}
 
 	r := MustTSMReader(dir, 1, writes)
@@ -1054,13 +1037,13 @@ func TestTSMKeyIterator_Duplicate(t *testing.T) {
 	v2 := tsm1.NewValue(1, int64(2))
 
 	writes1 := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v1},
+		"cpu,host=A#!~#value": {v1},
 	}
 
 	r1 := MustTSMReader(dir, 1, writes1)
 
 	writes2 := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v2},
+		"cpu,host=A#!~#value": {v2},
 	}
 
 	r2 := MustTSMReader(dir, 2, writes2)
@@ -1107,7 +1090,7 @@ func TestTSMKeyIterator_MultipleKeysDeleted(t *testing.T) {
 
 	v1 := tsm1.NewValue(2, int64(1))
 	points1 := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v1},
+		"cpu,host=A#!~#value": {v1},
 	}
 
 	r1 := MustTSMReader(dir, 1, points1)
@@ -1119,8 +1102,8 @@ func TestTSMKeyIterator_MultipleKeysDeleted(t *testing.T) {
 	v3 := tsm1.NewValue(1, float64(1))
 
 	points2 := map[string][]tsm1.Value{
-		"cpu,host=A#!~#count": []tsm1.Value{v2},
-		"cpu,host=B#!~#value": []tsm1.Value{v3},
+		"cpu,host=A#!~#count": {v2},
+		"cpu,host=B#!~#value": {v3},
 	}
 
 	r2 := MustTSMReader(dir, 2, points2)
@@ -1182,11 +1165,11 @@ func TestTSMKeyIterator_SingleDeletes(t *testing.T) {
 	v6 := tsm1.NewValue(60, int64(1))
 
 	points1 := map[string][]tsm1.Value{
-		"cpu,host=0#!~#value": []tsm1.Value{v1, v2},
-		"cpu,host=A#!~#value": []tsm1.Value{v5, v6},
-		"cpu,host=B#!~#value": []tsm1.Value{v3, v4},
-		"cpu,host=C#!~#value": []tsm1.Value{v1, v2},
-		"cpu,host=D#!~#value": []tsm1.Value{v1, v2},
+		"cpu,host=0#!~#value": {v1, v2},
+		"cpu,host=A#!~#value": {v5, v6},
+		"cpu,host=B#!~#value": {v3, v4},
+		"cpu,host=C#!~#value": {v1, v2},
+		"cpu,host=D#!~#value": {v1, v2},
 	}
 
 	r1 := MustTSMReader(dir, 1, points1)
@@ -1258,7 +1241,7 @@ func TestTSMKeyIterator_Abort(t *testing.T) {
 
 	v1 := tsm1.NewValue(1, 1.1)
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v1},
+		"cpu,host=A#!~#value": {v1},
 	}
 
 	r := MustTSMReader(dir, 1, writes)
@@ -1290,10 +1273,10 @@ func TestCacheKeyIterator_Single(t *testing.T) {
 	v0 := tsm1.NewValue(1, 1.0)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v0},
+		"cpu,host=A#!~#value": {v0},
 	}
 
-	c := tsm1.NewCache(0, "")
+	c := tsm1.NewCache(0)
 
 	for k, v := range writes {
 		if err := c.Write([]byte(k), v); err != nil {
@@ -1338,10 +1321,10 @@ func TestCacheKeyIterator_Chunked(t *testing.T) {
 	v1 := tsm1.NewValue(2, 2.0)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v0, v1},
+		"cpu,host=A#!~#value": {v0, v1},
 	}
 
-	c := tsm1.NewCache(0, "")
+	c := tsm1.NewCache(0)
 
 	for k, v := range writes {
 		if err := c.Write([]byte(k), v); err != nil {
@@ -1388,10 +1371,10 @@ func TestCacheKeyIterator_Abort(t *testing.T) {
 	v0 := tsm1.NewValue(1, 1.0)
 
 	writes := map[string][]tsm1.Value{
-		"cpu,host=A#!~#value": []tsm1.Value{v0},
+		"cpu,host=A#!~#value": {v0},
 	}
 
-	c := tsm1.NewCache(0, "")
+	c := tsm1.NewCache(0)
 
 	for k, v := range writes {
 		if err := c.Write([]byte(k), v); err != nil {
@@ -1425,15 +1408,15 @@ func TestDefaultPlanner_Plan_Min(t *testing.T) {
 		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return []tsm1.FileStat{
-					tsm1.FileStat{
+					{
 						Path: "01-01.tsm1",
 						Size: 1 * 1024 * 1024,
 					},
-					tsm1.FileStat{
+					{
 						Path: "02-01.tsm1",
 						Size: 1 * 1024 * 1024,
 					},
-					tsm1.FileStat{
+					{
 						Path: "03-1.tsm1",
 						Size: 251 * 1024 * 1024,
 					},
@@ -1452,31 +1435,31 @@ func TestDefaultPlanner_Plan_Min(t *testing.T) {
 // file that is in a larger step, the older ones will get compacted.
 func TestDefaultPlanner_Plan_CombineSequence(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 128 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-04.tsm1",
 			Size: 128 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 128 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 128 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-02.tsm1",
 			Size: 67 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-02.tsm1",
 			Size: 128 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "08-01.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
@@ -1506,39 +1489,39 @@ func TestDefaultPlanner_Plan_CombineSequence(t *testing.T) {
 // Ensure that the planner grabs the smallest compaction step
 func TestDefaultPlanner_Plan_MultipleGroups(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 129 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-04.tsm1",
 			Size: 129 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-04.tsm1",
 			Size: 129 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-04.tsm1",
 			Size: 129 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "08-04.tsm1",
 			Size: 129 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "09-04.tsm1", // should be skipped
 			Size: 129 * 1024 * 1024,
 		},
@@ -1583,51 +1566,51 @@ func TestDefaultPlanner_Plan_MultipleGroups(t *testing.T) {
 // Ensure that the planner grabs the smallest compaction step
 func TestDefaultPlanner_PlanLevel_SmallestCompactionStep(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-03.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "08-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "09-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "10-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "11-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "12-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1656,31 +1639,31 @@ func TestDefaultPlanner_PlanLevel_SmallestCompactionStep(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel_SplitFile(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-03.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-03.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1709,31 +1692,31 @@ func TestDefaultPlanner_PlanLevel_SplitFile(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel_IsolatedHighLevel(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-02.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-02.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-02.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-02.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-02.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1756,27 +1739,27 @@ func TestDefaultPlanner_PlanLevel_IsolatedHighLevel(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel3_MinFiles(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-03.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-01.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-01.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-02.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1799,16 +1782,16 @@ func TestDefaultPlanner_PlanLevel3_MinFiles(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel2_MinFiles(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "02-04.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
 
-		tsm1.FileStat{
+		{
 			Path: "03-02.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1831,28 +1814,28 @@ func TestDefaultPlanner_PlanLevel2_MinFiles(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel_Tombstone(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path:         "01-03.tsm1",
 			Size:         251 * 1024 * 1024,
 			HasTombstone: true,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-01.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-01.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-02.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1881,35 +1864,35 @@ func TestDefaultPlanner_PlanLevel_Tombstone(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel_Multiple(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-01.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-01.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-01.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "08-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -1939,67 +1922,67 @@ func TestDefaultPlanner_PlanLevel_Multiple(t *testing.T) {
 
 func TestDefaultPlanner_PlanLevel_InUse(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-01.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-01.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-01.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "08-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "09-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "10-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "11-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "12-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "13-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "14-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "15-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "16-01.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
@@ -2053,15 +2036,15 @@ func TestDefaultPlanner_PlanLevel_InUse(t *testing.T) {
 
 func TestDefaultPlanner_PlanOptimize_NoLevel4(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-03.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-03.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
@@ -2084,31 +2067,31 @@ func TestDefaultPlanner_PlanOptimize_NoLevel4(t *testing.T) {
 
 func TestDefaultPlanner_PlanOptimize_Level4(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-04.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
@@ -2141,43 +2124,43 @@ func TestDefaultPlanner_PlanOptimize_Level4(t *testing.T) {
 
 func TestDefaultPlanner_PlanOptimize_Multiple(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-03.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "07-04.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "08-04.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "09-04.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "10-04.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
@@ -2222,15 +2205,15 @@ func TestDefaultPlanner_PlanOptimize_Multiple(t *testing.T) {
 
 func TestDefaultPlanner_PlanOptimize_Optimized(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-03.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "01-05.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
@@ -2253,16 +2236,16 @@ func TestDefaultPlanner_PlanOptimize_Optimized(t *testing.T) {
 
 func TestDefaultPlanner_PlanOptimize_Tombstones(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 251 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path:         "01-05.tsm1",
 			Size:         1 * 1024 * 1024,
 			HasTombstone: true,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-06.tsm1",
 			Size: 2 * 1024 * 1024 * 1024,
 		},
@@ -2294,27 +2277,27 @@ func TestDefaultPlanner_PlanOptimize_Tombstones(t *testing.T) {
 // have happened in some interval
 func TestDefaultPlanner_Plan_FullOnCold(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-01.tsm1",
 			Size: 513 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-02.tsm1",
 			Size: 129 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-02.tsm1",
 			Size: 33 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-02.tsm1",
 			Size: 1 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-02.tsm1",
 			Size: 10 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "06-01.tsm1",
 			Size: 2 * 1024 * 1024,
 		},
@@ -2345,11 +2328,11 @@ func TestDefaultPlanner_Plan_FullOnCold(t *testing.T) {
 // allowable size
 func TestDefaultPlanner_Plan_SkipMaxSizeFiles(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-01.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-02.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
@@ -2373,19 +2356,19 @@ func TestDefaultPlanner_Plan_SkipMaxSizeFiles(t *testing.T) {
 // allowable size
 func TestDefaultPlanner_Plan_SkipPlanningAfterFull(t *testing.T) {
 	testSet := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-05.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-05.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-05.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
@@ -2408,23 +2391,23 @@ func TestDefaultPlanner_Plan_SkipPlanningAfterFull(t *testing.T) {
 
 	// skip planning if all files are over the limit
 	over := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
@@ -2465,43 +2448,43 @@ func TestDefaultPlanner_Plan_SkipPlanningAfterFull(t *testing.T) {
 // would get repeatedly plan the same files and never stop.
 func TestDefaultPlanner_Plan_TwoGenLevel3(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001666.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001667.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001668.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001669.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001670.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001671.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001672.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002245-000001673.tsm",
 			Size: 192631258,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002246-000000002.tsm",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "000002246-000000003.tsm",
 			Size: 192631258,
 		},
@@ -2526,19 +2509,19 @@ func TestDefaultPlanner_Plan_TwoGenLevel3(t *testing.T) {
 // size, but do not contain full blocks
 func TestDefaultPlanner_Plan_NotFullOverMaxsize(t *testing.T) {
 	testSet := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-05.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-05.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-05.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 256 * 1024 * 1024,
 		},
@@ -2565,11 +2548,11 @@ func TestDefaultPlanner_Plan_NotFullOverMaxsize(t *testing.T) {
 
 	// skip planning if all files are over the limit
 	over := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-05.tsm1",
 			Size: 2049 * 1024 * 1024,
 		},
@@ -2592,23 +2575,23 @@ func TestDefaultPlanner_Plan_NotFullOverMaxsize(t *testing.T) {
 // size even if there is a single file in the smaller step size
 func TestDefaultPlanner_Plan_CompactsMiddleSteps(t *testing.T) {
 	data := []tsm1.FileStat{
-		tsm1.FileStat{
+		{
 			Path: "01-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "02-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "03-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "04-04.tsm1",
 			Size: 64 * 1024 * 1024,
 		},
-		tsm1.FileStat{
+		{
 			Path: "05-02.tsm1",
 			Size: 2 * 1024 * 1024,
 		},
@@ -2640,23 +2623,23 @@ func TestDefaultPlanner_Plan_LargeGeneration(t *testing.T) {
 		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return []tsm1.FileStat{
-					tsm1.FileStat{
+					{
 						Path: "000000278-000000006.tsm",
 						Size: 2148340232,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000278-000000007.tsm",
 						Size: 2148356556,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000278-000000008.tsm",
 						Size: 167780181,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000278-000047040.tsm",
 						Size: 2148728539,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000278-000047041.tsm",
 						Size: 701863692,
 					},
@@ -2676,55 +2659,55 @@ func TestDefaultPlanner_Plan_ForceFull(t *testing.T) {
 		&fakeFileStore{
 			PathsFn: func() []tsm1.FileStat {
 				return []tsm1.FileStat{
-					tsm1.FileStat{
+					{
 						Path: "000000001-000000001.tsm",
 						Size: 2148340232,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000002-000000001.tsm",
 						Size: 2148356556,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000003-000000001.tsm",
 						Size: 167780181,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000004-000000001.tsm",
 						Size: 2148728539,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000005-000000001.tsm",
 						Size: 2148340232,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000006-000000001.tsm",
 						Size: 2148356556,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000007-000000001.tsm",
 						Size: 167780181,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000008-000000001.tsm",
 						Size: 2148728539,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000009-000000002.tsm",
 						Size: 701863692,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000010-000000002.tsm",
 						Size: 701863692,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000011-000000002.tsm",
 						Size: 701863692,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000012-000000002.tsm",
 						Size: 701863692,
 					},
-					tsm1.FileStat{
+					{
 						Path: "000000013-000000002.tsm",
 						Size: 701863692,
 					},
@@ -2804,7 +2787,7 @@ func MustTSMWriter(dir string, gen int) (tsm1.TSMWriter, string) {
 		panic(fmt.Sprintf("close temp file: %v", err))
 	}
 
-	newName := filepath.Join(filepath.Dir(oldName), tsmFileName(gen))
+	newName := filepath.Join(filepath.Dir(oldName), tsm1.DefaultFormatFileName(gen, 1)+".tsm")
 	if err := os.Rename(oldName, newName); err != nil {
 		panic(fmt.Sprintf("create tsm file: %v", err))
 	}
@@ -2892,6 +2875,7 @@ func (w *fakeFileStore) BlockCount(path string, idx int) int {
 func (w *fakeFileStore) TSMReader(path string) *tsm1.TSMReader {
 	r := MustOpenTSMReader(path)
 	w.readers = append(w.readers, r)
+	r.Ref()
 	return r
 }
 
