@@ -1,29 +1,23 @@
 package tsm1
 
 import (
-	"compress/gzip"
 	"encoding/binary"
 	"encoding/json"
 	"io"
-)
 
-type writeFlushCloser interface {
-	Close() error
-	Write(b []byte) (int, error)
-	Flush() error
-}
+	"github.com/golang/snappy"
+)
 
 // DigestWriter allows for writing a digest of a shard.  A digest is a condensed
 // representation of the contents of a shard.  It can be scoped to one or more series
 // keys, ranges of times or sets of files.
 type DigestWriter struct {
-	w io.WriteCloser
-	F writeFlushCloser
+	w  io.WriteCloser
+	sw *snappy.Writer
 }
 
 func NewDigestWriter(w io.WriteCloser) (*DigestWriter, error) {
-	gw := gzip.NewWriter(w)
-	return &DigestWriter{w: w, F: gw}, nil
+	return &DigestWriter{w: w, sw: snappy.NewBufferedWriter(w)}, nil
 }
 
 func (w *DigestWriter) WriteManifest(m *DigestManifest) error {
@@ -33,42 +27,42 @@ func (w *DigestWriter) WriteManifest(m *DigestManifest) error {
 	}
 
 	// Write length of manifest.
-	if err := binary.Write(w.F, binary.BigEndian, uint32(len(b))); err != nil {
+	if err := binary.Write(w.sw, binary.BigEndian, uint32(len(b))); err != nil {
 		return err
 	}
 
 	// Write manifest.
-	_, err = w.F.Write(b)
+	_, err = w.sw.Write(b)
 	return err
 }
 
 func (w *DigestWriter) WriteTimeSpan(key string, t *DigestTimeSpan) error {
-	if err := binary.Write(w.F, binary.BigEndian, uint16(len(key))); err != nil {
+	if err := binary.Write(w.sw, binary.BigEndian, uint16(len(key))); err != nil {
 		return err
 	}
 
-	if _, err := w.F.Write([]byte(key)); err != nil {
+	if _, err := w.sw.Write([]byte(key)); err != nil {
 		return err
 	}
 
-	if err := binary.Write(w.F, binary.BigEndian, uint32(t.Len())); err != nil {
+	if err := binary.Write(w.sw, binary.BigEndian, uint32(t.Len())); err != nil {
 		return err
 	}
 
 	for _, tr := range t.Ranges {
-		if err := binary.Write(w.F, binary.BigEndian, tr.Min); err != nil {
+		if err := binary.Write(w.sw, binary.BigEndian, tr.Min); err != nil {
 			return err
 		}
 
-		if err := binary.Write(w.F, binary.BigEndian, tr.Max); err != nil {
+		if err := binary.Write(w.sw, binary.BigEndian, tr.Max); err != nil {
 			return err
 		}
 
-		if err := binary.Write(w.F, binary.BigEndian, tr.CRC); err != nil {
+		if err := binary.Write(w.sw, binary.BigEndian, tr.CRC); err != nil {
 			return err
 		}
 
-		if err := binary.Write(w.F, binary.BigEndian, uint16(tr.N)); err != nil {
+		if err := binary.Write(w.sw, binary.BigEndian, uint16(tr.N)); err != nil {
 			return err
 		}
 	}
@@ -77,7 +71,7 @@ func (w *DigestWriter) WriteTimeSpan(key string, t *DigestTimeSpan) error {
 }
 
 func (w *DigestWriter) Flush() error {
-	return w.F.Flush()
+	return w.sw.Flush()
 }
 
 func (w *DigestWriter) Close() error {
@@ -85,7 +79,7 @@ func (w *DigestWriter) Close() error {
 		return err
 	}
 
-	if err := w.F.Close(); err != nil {
+	if err := w.sw.Close(); err != nil {
 		return err
 	}
 
