@@ -597,13 +597,13 @@ func BenchmarkIntegerEncoderPackedSimple(b *testing.B) {
 	b.ResetTimer()
 	for i := 0; i < b.N; i++ {
 		enc.Bytes()
+		enc.Reset()
+		for i := 0; i < len(x); i++ {
+			enc.Write(x[i])
+		}
 	}
 }
 
-type byteSetter interface {
-	SetBytes(b []byte)
-}
-
 func BenchmarkIntegerDecoderPackedSimple(b *testing.B) {
 	x := make([]int64, 1024)
 	enc := NewIntegerEncoder(1024)
