@@ -17,6 +17,12 @@ import (
 	"go.uber.org/zap"
 )
 
+// Available index types.
+const (
+	InmemIndexName = "inmem"
+	TSI1IndexName  = "tsi1"
+)
+
 type Index interface {
 	Open() error
 	Close() error
@@ -34,6 +40,9 @@ type Index interface {
 	DropSeries(seriesID uint64, key []byte, cascade bool) error
 	DropMeasurementIfSeriesNotExist(name []byte) error
 
+	// Used to clean up series in inmem index that were dropped with a shard.
+	DropSeriesGlobal(key []byte) error
+
 	MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error)
 	SeriesN() int64
 	SeriesSketches() (estimator.Sketch, estimator.Sketch, error)
@@ -272,6 +281,17 @@ func (a SeriesIDIterators) Close() (err error) {
 	return err
 }
 
+func (a SeriesIDIterators) filterNonNil() []SeriesIDIterator {
+	other := make([]SeriesIDIterator, 0, len(a))
+	for _, itr := range a {
+		if itr == nil {
+			continue
+		}
+		other = append(other, itr)
+	}
+	return other
+}
+
 // seriesQueryAdapterIterator adapts SeriesIDIterator to an influxql.Iterator.
 type seriesQueryAdapterIterator struct {
 	once     sync.Once
@@ -414,6 +434,7 @@ func MergeSeriesIDIterators(itrs ...SeriesIDIterator) SeriesIDIterator {
 	} else if n == 1 {
 		return itrs[0]
 	}
+	itrs = SeriesIDIterators(itrs).filterNonNil()
 
 	// Merge as series id sets, if available.
 	if a := NewSeriesIDSetIterators(itrs); a != nil {
@@ -1207,7 +1228,7 @@ type IndexSet struct {
 // HasInmemIndex returns true if any in-memory index is in use.
 func (is IndexSet) HasInmemIndex() bool {
 	for _, idx := range is.Indexes {
-		if idx.Type() == "inmem" {
+		if idx.Type() == InmemIndexName {
 			return true
 		}
 	}
@@ -2523,6 +2544,11 @@ func (is IndexSet) TagSets(sfile *SeriesFile, name []byte, opt query.IteratorOpt
 		maxSeriesN = int(^uint(0) >> 1)
 	}
 
+	// The tag sets require a string for each series key in the set, The series
+	// file formatted keys need to be parsed into models format. Since they will
+	// end up as strings we can re-use an intermediate buffer for this process.
+	var keyBuf []byte
+	var tagsBuf models.Tags // Buffer for tags. Tags are not needed outside of each loop iteration.
 	for {
 		se, err := itr.Next()
 		if err != nil {
@@ -2550,14 +2576,15 @@ func (is IndexSet) TagSets(sfile *SeriesFile, name []byte, opt query.IteratorOpt
 			return nil, fmt.Errorf("max-select-series limit exceeded: (%d/%d)", seriesN, opt.MaxSeriesN)
 		}
 
-		_, tags := ParseSeriesKey(key)
-		if opt.Authorizer != nil && !opt.Authorizer.AuthorizeSeriesRead(db, name, tags) {
+		// NOTE - must not escape this loop iteration.
+		_, tagsBuf = ParseSeriesKeyInto(key, tagsBuf)
+		if opt.Authorizer != nil && !opt.Authorizer.AuthorizeSeriesRead(db, name, tagsBuf) {
 			continue
 		}
 
 		var tagsAsKey []byte
 		if len(dims) > 0 {
-			tagsAsKey = MakeTagsKey(dims, tags)
+			tagsAsKey = MakeTagsKey(dims, tagsBuf)
 		}
 
 		tagSet, ok := tagSets[string(tagsAsKey)]
@@ -2570,7 +2597,9 @@ func (is IndexSet) TagSets(sfile *SeriesFile, name []byte, opt query.IteratorOpt
 		}
 
 		// Associate the series and filter with the Tagset.
-		tagSet.AddFilter(string(models.MakeKey(name, tags)), se.Expr)
+		keyBuf = models.AppendMakeKey(keyBuf, name, tagsBuf)
+		tagSet.AddFilter(string(keyBuf), se.Expr)
+		keyBuf = keyBuf[:0]
 
 		// Ensure it's back in the map.
 		tagSets[string(tagsAsKey)] = tagSet
@@ -2640,7 +2669,7 @@ func NewIndex(id uint64, database, path string, seriesIDSet *SeriesIDSet, sfile
 	} else if err != nil {
 		return nil, err
 	} else if err == nil {
-		format = "tsi1"
+		format = TSI1IndexName
 	}
 
 	// Lookup index by format.
