@@ -6,9 +6,11 @@ import (
 	"regexp"
 	"sort"
 	"sync"
+	"unsafe"
 
 	"github.com/influxdata/influxdb/models"
 	"github.com/influxdata/influxdb/pkg/bytesutil"
+	"github.com/influxdata/influxdb/pkg/radix"
 	"github.com/influxdata/influxdb/query"
 	"github.com/influxdata/influxdb/tsdb"
 	"github.com/influxdata/influxql"
@@ -51,6 +53,40 @@ func newMeasurement(database, name string) *measurement {
 	}
 }
 
+// bytes estimates the memory footprint of this measurement, in bytes.
+func (m *measurement) bytes() int {
+	var b int
+	m.mu.RLock()
+	b += int(unsafe.Sizeof(m.Database)) + len(m.Database)
+	b += int(unsafe.Sizeof(m.Name)) + len(m.Name)
+	if m.NameBytes != nil {
+		b += int(unsafe.Sizeof(m.NameBytes)) + len(m.NameBytes)
+	}
+	b += 24 // 24 bytes for m.mu RWMutex
+	b += int(unsafe.Sizeof(m.fieldNames))
+	for fieldName := range m.fieldNames {
+		b += int(unsafe.Sizeof(fieldName)) + len(fieldName)
+	}
+	b += int(unsafe.Sizeof(m.seriesByID))
+	for k, v := range m.seriesByID {
+		b += int(unsafe.Sizeof(k))
+		b += int(unsafe.Sizeof(v))
+		// Do not count footprint of each series, to avoid double-counting in Index.bytes().
+	}
+	b += int(unsafe.Sizeof(m.seriesByTagKeyValue))
+	for k, v := range m.seriesByTagKeyValue {
+		b += int(unsafe.Sizeof(k)) + len(k)
+		b += int(unsafe.Sizeof(v)) + v.bytes()
+	}
+	b += int(unsafe.Sizeof(m.sortedSeriesIDs))
+	for _, seriesID := range m.sortedSeriesIDs {
+		b += int(unsafe.Sizeof(seriesID))
+	}
+	b += int(unsafe.Sizeof(m.dirty))
+	m.mu.RUnlock()
+	return b
+}
+
 // Authorized determines if this Measurement is authorized to be read, according
 // to the provided Authorizer. A measurement is authorized to be read if at
 // least one undeleted series from the measurement is authorized to be read.
@@ -210,19 +246,6 @@ func (m *measurement) HasSeries() bool {
 	return len(m.seriesByID) > 0
 }
 
-// Cardinality returns the number of values associated with the given tag key.
-func (m *measurement) Cardinality(key string) int {
-	var n int
-	m.mu.RLock()
-	n = m.cardinality(key)
-	m.mu.RUnlock()
-	return n
-}
-
-func (m *measurement) cardinality(key string) int {
-	return m.seriesByTagKeyValue[key].Cardinality()
-}
-
 // CardinalityBytes returns the number of values associated with the given tag key.
 func (m *measurement) CardinalityBytes(key []byte) int {
 	m.mu.RLock()
@@ -338,25 +361,6 @@ func (m *measurement) filters(condition influxql.Expr) ([]uint64, map[uint64]inf
 	return m.WalkWhereForSeriesIds(condition)
 }
 
-// ForEachSeriesByExpr iterates over all series filtered by condition.
-func (m *measurement) ForEachSeriesByExpr(condition influxql.Expr, fn func(tags models.Tags) error) error {
-	// Retrieve matching series ids.
-	ids, _, err := m.filters(condition)
-	if err != nil {
-		return err
-	}
-
-	// Iterate over each series.
-	for _, id := range ids {
-		s := m.SeriesByID(id)
-		if err := fn(s.Tags); err != nil {
-			return err
-		}
-	}
-
-	return nil
-}
-
 // TagSets returns the unique tag sets that exist for the given tag keys. This is used to determine
 // what composite series will be created by a group by. i.e. "group by region" should return:
 // {"region":"uswest"}, {"region":"useast"}
@@ -632,7 +636,9 @@ func (m *measurement) idsForExpr(n *influxql.BinaryExpr) (seriesIDs, influxql.Ex
 	if !ok {
 		name, ok = n.RHS.(*influxql.VarRef)
 		if !ok {
-			return nil, nil, fmt.Errorf("invalid expression: %s", n.String())
+			// This is an expression we do not know how to evaluate. Let the
+			// query engine take care of this.
+			return m.SeriesIDs(), n, nil
 		}
 		value = n.LHS
 	}
@@ -682,7 +688,10 @@ func (m *measurement) idsForExpr(n *influxql.BinaryExpr) (seriesIDs, influxql.Ex
 			}
 		} else if n.Op == influxql.NEQ {
 			if str.Val != "" {
-				ids = m.SeriesIDs().Reject(tagVals.Load(str.Val))
+				ids = m.SeriesIDs()
+				if vals := tagVals.Load(str.Val); len(vals) > 0 {
+					ids = ids.Reject(vals)
+				}
 			} else {
 				tagVals.RangeAll(func(_ string, a seriesIDs) {
 					ids = append(ids, a...)
@@ -771,10 +780,9 @@ func (m *measurement) idsForExpr(n *influxql.BinaryExpr) (seriesIDs, influxql.Ex
 		return ids, nil, nil
 	}
 
-	if n.Op == influxql.NEQ || n.Op == influxql.NEQREGEX {
-		return m.SeriesIDs(), nil, nil
-	}
-	return nil, nil, nil
+	// We do not know how to evaluate this expression so pass it
+	// on to the query engine.
+	return m.SeriesIDs(), n, nil
 }
 
 // FilterExprs represents a map of series IDs to filter expressions.
@@ -996,63 +1004,6 @@ func (a measurements) Less(i, j int) bool { return a[i].Name < a[j].Name }
 // Swap implements sort.Interface.
 func (a measurements) Swap(i, j int) { a[i], a[j] = a[j], a[i] }
 
-func (a measurements) Intersect(other measurements) measurements {
-	l := a
-	r := other
-
-	// we want to iterate through the shortest one and stop
-	if len(other) < len(a) {
-		l = other
-		r = a
-	}
-
-	// they're in sorted order so advance the counter as needed.
-	// That is, don't run comparisons against lower values that we've already passed
-	var i, j int
-
-	result := make(measurements, 0, len(l))
-	for i < len(l) && j < len(r) {
-		if l[i].Name == r[j].Name {
-			result = append(result, l[i])
-			i++
-			j++
-		} else if l[i].Name < r[j].Name {
-			i++
-		} else {
-			j++
-		}
-	}
-
-	return result
-}
-
-func (a measurements) Union(other measurements) measurements {
-	result := make(measurements, 0, len(a)+len(other))
-	var i, j int
-	for i < len(a) && j < len(other) {
-		if a[i].Name == other[j].Name {
-			result = append(result, a[i])
-			i++
-			j++
-		} else if a[i].Name < other[j].Name {
-			result = append(result, a[i])
-			i++
-		} else {
-			result = append(result, other[j])
-			j++
-		}
-	}
-
-	// now append the remainder
-	if i < len(a) {
-		result = append(result, a[i:]...)
-	} else if j < len(other) {
-		result = append(result, other[j:]...)
-	}
-
-	return result
-}
-
 // series belong to a Measurement and represent unique time series in a database.
 type series struct {
 	mu      sync.RWMutex
@@ -1075,6 +1026,22 @@ func newSeries(id uint64, m *measurement, key string, tags models.Tags) *series
 	}
 }
 
+// bytes estimates the memory footprint of this series, in bytes.
+func (s *series) bytes() int {
+	var b int
+	s.mu.RLock()
+	b += 24 // RWMutex uses 24 bytes
+	b += int(unsafe.Sizeof(s.deleted) + unsafe.Sizeof(s.ID))
+	// Do not count s.Measurement to prevent double-counting in Index.Bytes.
+	b += int(unsafe.Sizeof(s.Key)) + len(s.Key)
+	for _, tag := range s.Tags {
+		b += int(unsafe.Sizeof(tag)) + len(tag.Key) + len(tag.Value)
+	}
+	b += int(unsafe.Sizeof(s.Tags))
+	s.mu.RUnlock()
+	return b
+}
+
 // Delete marks this series as deleted.  A deleted series should not be returned for queries.
 func (s *series) Delete() {
 	s.mu.Lock()
@@ -1092,13 +1059,27 @@ func (s *series) Deleted() bool {
 
 // TagKeyValue provides goroutine-safe concurrent access to the set of series
 // ids mapping to a set of tag values.
-//
-// TODO(edd): This could possibly be replaced by a sync.Map once we use Go 1.9.
 type tagKeyValue struct {
 	mu      sync.RWMutex
 	entries map[string]*tagKeyValueEntry
 }
 
+// bytes estimates the memory footprint of this tagKeyValue, in bytes.
+func (t *tagKeyValue) bytes() int {
+	var b int
+	t.mu.RLock()
+	b += 24 // RWMutex is 24 bytes
+	b += int(unsafe.Sizeof(t.entries))
+	for k, v := range t.entries {
+		b += int(unsafe.Sizeof(k)) + len(k)
+		b += len(v.m) * 8 // uint64
+		b += len(v.a) * 8 // uint64
+		b += int(unsafe.Sizeof(v) + unsafe.Sizeof(*v))
+	}
+	t.mu.RUnlock()
+	return b
+}
+
 // NewTagKeyValue initialises a new TagKeyValue.
 func newTagKeyValue() *tagKeyValue {
 	return &tagKeyValue{entries: make(map[string]*tagKeyValueEntry)}
@@ -1127,18 +1108,6 @@ func (t *tagKeyValue) Contains(value string) bool {
 	return ok
 }
 
-// InsertSeriesID adds a series id to the tag key value.
-func (t *tagKeyValue) InsertSeriesID(value string, id uint64) {
-	t.mu.Lock()
-	entry := t.entries[value]
-	if entry == nil {
-		entry = newTagKeyValueEntry()
-		t.entries[value] = entry
-	}
-	entry.m[id] = struct{}{}
-	t.mu.Unlock()
-}
-
 // InsertSeriesIDByte adds a series id to the tag key value.
 func (t *tagKeyValue) InsertSeriesIDByte(value []byte, id uint64) {
 	t.mu.Lock()
@@ -1164,21 +1133,6 @@ func (t *tagKeyValue) Load(value string) seriesIDs {
 	return ids
 }
 
-// LoadByte returns the SeriesIDs for the provided tag value. It makes use of
-// Go's compiler optimisation for avoiding a copy when accessing maps with a []byte.
-func (t *tagKeyValue) LoadByte(value []byte) seriesIDs {
-	if t == nil {
-		return nil
-	}
-
-	t.mu.RLock()
-	entry := t.entries[string(value)]
-	ids := entry.ids()
-	t.mu.RUnlock()
-	return ids
-}
-
-// Range calls f sequentially on each key and value. A call to Range on a nil
 // TagKeyValue is a no-op.
 //
 // If f returns false then iteration over any remaining keys or values will cease.
@@ -1228,7 +1182,7 @@ func (e *tagKeyValueEntry) ids() seriesIDs {
 	for id := range e.m {
 		a = append(a, id)
 	}
-	sort.Sort(a)
+	radix.SortUint64s(a)
 
 	e.a = a
 	return e.a
@@ -1438,17 +1392,6 @@ type TagFilter struct {
 	Regex *regexp.Regexp
 }
 
-// WalkTagKeys calls fn for each tag key associated with m.  The order of the
-// keys is undefined.
-func (m *measurement) WalkTagKeys(fn func(k string)) {
-	m.mu.RLock()
-	defer m.mu.RUnlock()
-
-	for k := range m.seriesByTagKeyValue {
-		fn(k)
-	}
-}
-
 // TagKeys returns a list of the measurement's tag names, in sorted order.
 func (m *measurement) TagKeys() []string {
 	m.mu.RLock()
@@ -1501,18 +1444,6 @@ func (m *measurement) SetFieldName(name string) {
 	m.mu.Unlock()
 }
 
-// FieldNames returns a list of the measurement's field names, in an arbitrary order.
-func (m *measurement) FieldNames() []string {
-	m.mu.RLock()
-	defer m.mu.RUnlock()
-
-	a := make([]string, 0, len(m.fieldNames))
-	for n := range m.fieldNames {
-		a = append(a, n)
-	}
-	return a
-}
-
 // SeriesByTagKeyValue returns the TagKeyValue for the provided tag key.
 func (m *measurement) SeriesByTagKeyValue(key string) *tagKeyValue {
 	m.mu.RLock()
