@@ -24,7 +24,7 @@ import (
 )
 
 // IndexName is the name of the index.
-const IndexName = "tsi1"
+const IndexName = tsdb.TSI1IndexName
 
 // ErrCompactionInterrupted is returned if compactions are disabled or
 // an index is closed while a compaction is occurring.
@@ -128,6 +128,10 @@ type Index struct {
 	sfile    *tsdb.SeriesFile // series lookup file
 	database string           // Name of database.
 
+	// Cached sketches.
+	mSketch, mTSketch estimator.Sketch // Measurement sketches
+	sSketch, sTSketch estimator.Sketch // Series sketches
+
 	// Index's version.
 	version int
 
@@ -147,6 +151,10 @@ func NewIndex(sfile *tsdb.SeriesFile, database string, options ...IndexOption) *
 		version:        Version,
 		sfile:          sfile,
 		database:       database,
+		mSketch:        hll.NewDefaultPlus(),
+		mTSketch:       hll.NewDefaultPlus(),
+		sSketch:        hll.NewDefaultPlus(),
+		sTSketch:       hll.NewDefaultPlus(),
 		PartitionN:     DefaultPartitionN,
 	}
 
@@ -173,6 +181,10 @@ func (i *Index) Bytes() int {
 	b += int(unsafe.Sizeof(i.logger))
 	b += int(unsafe.Sizeof(i.sfile))
 	// Do not count SeriesFile because it belongs to the code that constructed this Index.
+	b += int(unsafe.Sizeof(i.mSketch)) + i.mSketch.Bytes()
+	b += int(unsafe.Sizeof(i.mTSketch)) + i.mTSketch.Bytes()
+	b += int(unsafe.Sizeof(i.sSketch)) + i.sSketch.Bytes()
+	b += int(unsafe.Sizeof(i.sTSketch)) + i.sTSketch.Bytes()
 	b += int(unsafe.Sizeof(i.database)) + len(i.database)
 	b += int(unsafe.Sizeof(i.version))
 	b += int(unsafe.Sizeof(i.PartitionN))
@@ -267,6 +279,13 @@ func (i *Index) Open() error {
 		}
 	}
 
+	// Refresh cached sketches.
+	if err := i.updateSeriesSketches(); err != nil {
+		return err
+	} else if err := i.updateMeasurementSketches(); err != nil {
+		return err
+	}
+
 	// Mark opened.
 	i.opened = true
 	i.logger.Info(fmt.Sprintf("index opened with %d partitions", partitionN))
@@ -346,6 +365,36 @@ func (i *Index) availableThreads() int {
 	return n
 }
 
+// updateMeasurementSketches rebuilds the cached measurement sketches.
+func (i *Index) updateMeasurementSketches() error {
+	i.mSketch, i.mTSketch = hll.NewDefaultPlus(), hll.NewDefaultPlus()
+	for j := 0; j < int(i.PartitionN); j++ {
+		if s, t, err := i.partitions[j].MeasurementsSketches(); err != nil {
+			return err
+		} else if i.mSketch.Merge(s); err != nil {
+			return err
+		} else if i.mTSketch.Merge(t); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+// updateSeriesSketches rebuilds the cached series sketches.
+func (i *Index) updateSeriesSketches() error {
+	i.sSketch, i.sTSketch = hll.NewDefaultPlus(), hll.NewDefaultPlus()
+	for j := 0; j < int(i.PartitionN); j++ {
+		if s, t, err := i.partitions[j].SeriesSketches(); err != nil {
+			return err
+		} else if i.sSketch.Merge(s); err != nil {
+			return err
+		} else if i.sTSketch.Merge(t); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 // SetFieldSet sets a shared field set from the engine.
 func (i *Index) SetFieldSet(fs *tsdb.MeasurementFieldSet) {
 	for _, p := range i.partitions {
@@ -554,6 +603,16 @@ func (i *Index) DropMeasurement(name []byte) error {
 			return err
 		}
 	}
+
+	// Update sketches under lock.
+	i.mu.Lock()
+	defer i.mu.Unlock()
+
+	i.mTSketch.Add(name)
+	if err := i.updateSeriesSketches(); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -601,12 +660,32 @@ func (i *Index) CreateSeriesListIfNotExists(keys [][]byte, names [][]byte, tagsS
 			return err
 		}
 	}
+
+	// Update sketches under lock.
+	i.mu.Lock()
+	defer i.mu.Unlock()
+
+	for _, key := range keys {
+		i.sSketch.Add(key)
+	}
+	for _, name := range names {
+		i.mSketch.Add(name)
+	}
+
 	return nil
 }
 
 // CreateSeriesIfNotExists creates a series if it doesn't exist or is deleted.
 func (i *Index) CreateSeriesIfNotExists(key, name []byte, tags models.Tags) error {
-	return i.partition(key).createSeriesListIfNotExists([][]byte{name}, []models.Tags{tags})
+	if err := i.partition(key).createSeriesListIfNotExists([][]byte{name}, []models.Tags{tags}); err != nil {
+		return err
+	}
+
+	i.mu.Lock()
+	i.sSketch.Add(key)
+	i.mSketch.Add(name)
+	i.mu.Unlock()
+	return nil
 }
 
 // InitializeSeries is a no-op. This only applies to the in-memory index.
@@ -622,6 +701,11 @@ func (i *Index) DropSeries(seriesID uint64, key []byte, cascade bool) error {
 		return err
 	}
 
+	// Add sketch tombstone.
+	i.mu.Lock()
+	i.sTSketch.Add(key)
+	i.mu.Unlock()
+
 	if !cascade {
 		return nil
 	}
@@ -643,6 +727,9 @@ func (i *Index) DropSeries(seriesID uint64, key []byte, cascade bool) error {
 	return nil
 }
 
+// DropSeriesGlobal is a no-op on the tsi1 index.
+func (i *Index) DropSeriesGlobal(key []byte) error { return nil }
+
 // DropMeasurementIfSeriesNotExist drops a measurement only if there are no more
 // series for the measurment.
 func (i *Index) DropMeasurementIfSeriesNotExist(name []byte) error {
@@ -657,46 +744,18 @@ func (i *Index) DropMeasurementIfSeriesNotExist(name []byte) error {
 	return i.DropMeasurement(name)
 }
 
-// MeasurementsSketches returns the two sketches for the index by merging all
-// instances of the type sketch types in all the partitions.
+// MeasurementsSketches returns the two measurement sketches for the index.
 func (i *Index) MeasurementsSketches() (estimator.Sketch, estimator.Sketch, error) {
-	s, ts := hll.NewDefaultPlus(), hll.NewDefaultPlus()
-	for _, p := range i.partitions {
-		// Get partition's measurement sketches and merge.
-		ps, pts, err := p.MeasurementsSketches()
-		if err != nil {
-			return nil, nil, err
-		}
-
-		if err := s.Merge(ps); err != nil {
-			return nil, nil, err
-		} else if err := ts.Merge(pts); err != nil {
-			return nil, nil, err
-		}
-	}
-
-	return s, ts, nil
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+	return i.mSketch.Clone(), i.mTSketch.Clone(), nil
 }
 
-// SeriesSketches returns the two sketches for the index by merging all
-// instances of the type sketch types in all the partitions.
+// SeriesSketches returns the two series sketches for the index.
 func (i *Index) SeriesSketches() (estimator.Sketch, estimator.Sketch, error) {
-	s, ts := hll.NewDefaultPlus(), hll.NewDefaultPlus()
-	for _, p := range i.partitions {
-		// Get partition's measurement sketches and merge.
-		ps, pts, err := p.SeriesSketches()
-		if err != nil {
-			return nil, nil, err
-		}
-
-		if err := s.Merge(ps); err != nil {
-			return nil, nil, err
-		} else if err := ts.Merge(pts); err != nil {
-			return nil, nil, err
-		}
-	}
-
-	return s, ts, nil
+	i.mu.RLock()
+	defer i.mu.RUnlock()
+	return i.sSketch.Clone(), i.sTSketch.Clone(), nil
 }
 
 // Since indexes are not shared across shards, the count returned by SeriesN
