@@ -57,8 +57,8 @@ type IndexFile struct {
 	seriesIDSetData          []byte
 	tombstoneSeriesIDSetData []byte
 
-	// Series sketches
-	sketch, tSketch estimator.Sketch
+	// Series sketch data.
+	sketchData, tSketchData []byte
 
 	// Sortable identifier & filepath to the log file.
 	level int
@@ -75,9 +75,7 @@ type IndexFile struct {
 // NewIndexFile returns a new instance of IndexFile.
 func NewIndexFile(sfile *tsdb.SeriesFile) *IndexFile {
 	return &IndexFile{
-		sfile:   sfile,
-		sketch:  hll.NewDefaultPlus(),
-		tSketch: hll.NewDefaultPlus(),
+		sfile: sfile,
 	}
 }
 
@@ -99,8 +97,6 @@ func (f *IndexFile) bytes() int {
 	b += int(unsafe.Sizeof(f.mblk)) + f.mblk.bytes()
 	b += int(unsafe.Sizeof(f.seriesIDSetData) + unsafe.Sizeof(f.tombstoneSeriesIDSetData))
 	// Do not count contents of seriesIDSetData or tombstoneSeriesIDSetData: references f.data
-	b += int(unsafe.Sizeof(f.sketch)) + f.sketch.Bytes()
-	b += int(unsafe.Sizeof(f.tSketch)) + f.tSketch.Bytes()
 	b += int(unsafe.Sizeof(f.level) + unsafe.Sizeof(f.id))
 	b += 24 // mu RWMutex is 24 bytes
 	b += int(unsafe.Sizeof(f.compacting))
@@ -186,26 +182,15 @@ func (f *IndexFile) UnmarshalBinary(data []byte) error {
 	}
 
 	// Slice series sketch data.
-	buf := data[t.SeriesSketch.Offset : t.SeriesSketch.Offset+t.SeriesSketch.Size]
-	if err := f.sketch.UnmarshalBinary(buf); err != nil {
-		return err
-	}
-
-	buf = data[t.TombstoneSeriesSketch.Offset : t.TombstoneSeriesSketch.Offset+t.TombstoneSeriesSketch.Size]
-	if err := f.tSketch.UnmarshalBinary(buf); err != nil {
-		return err
-	}
+	f.sketchData = data[t.SeriesSketch.Offset : t.SeriesSketch.Offset+t.SeriesSketch.Size]
+	f.tSketchData = data[t.TombstoneSeriesSketch.Offset : t.TombstoneSeriesSketch.Offset+t.TombstoneSeriesSketch.Size]
 
 	// Slice series set data.
 	f.seriesIDSetData = data[t.SeriesIDSet.Offset : t.SeriesIDSet.Offset+t.SeriesIDSet.Size]
 	f.tombstoneSeriesIDSetData = data[t.TombstoneSeriesIDSet.Offset : t.TombstoneSeriesIDSet.Offset+t.TombstoneSeriesIDSet.Size]
 
-	// Slice measurement block data.
-	buf = data[t.MeasurementBlock.Offset:]
-	buf = buf[:t.MeasurementBlock.Size]
-
 	// Unmarshal measurement block.
-	if err := f.mblk.UnmarshalBinary(buf); err != nil {
+	if err := f.mblk.UnmarshalBinary(data[t.MeasurementBlock.Offset:][:t.MeasurementBlock.Size]); err != nil {
 		return err
 	}
 
@@ -398,22 +383,23 @@ func (f *IndexFile) MeasurementSeriesIDIterator(name []byte) tsdb.SeriesIDIterat
 	return f.mblk.SeriesIDIterator(name)
 }
 
-// MergeMeasurementsSketches merges the index file's measurements sketches into
-// the provided sketches.
-func (f *IndexFile) MergeMeasurementsSketches(s, t estimator.Sketch) error {
-	if err := s.Merge(f.mblk.sketch); err != nil {
-		return err
-	}
-	return t.Merge(f.mblk.tSketch)
+// MeasurementsSketches returns existence and tombstone sketches for measurements.
+func (f *IndexFile) MeasurementsSketches() (sketch, tSketch estimator.Sketch, err error) {
+	return f.mblk.Sketches()
 }
 
-// MergeSeriesSketches merges the index file's series sketches into the provided
-// sketches.
-func (f *IndexFile) MergeSeriesSketches(s, t estimator.Sketch) error {
-	if err := s.Merge(f.sketch); err != nil {
-		return err
+// SeriesSketches returns existence and tombstone sketches for series.
+func (f *IndexFile) SeriesSketches() (sketch, tSketch estimator.Sketch, err error) {
+	sketch = hll.NewDefaultPlus()
+	if err := sketch.UnmarshalBinary(f.sketchData); err != nil {
+		return nil, nil, err
 	}
-	return t.Merge(f.tSketch)
+
+	tSketch = hll.NewDefaultPlus()
+	if err := tSketch.UnmarshalBinary(f.tSketchData); err != nil {
+		return nil, nil, err
+	}
+	return sketch, tSketch, nil
 }
 
 // ReadIndexFileTrailer returns the index file trailer from data.
